@@ -23,11 +23,6 @@
 
 #include <cogl/cogl.h>
 
-CoglTexture * meta_create_color_texture_4ub (guint8           red,
-                                             guint8           green,
-                                             guint8           blue,
-                                             guint8           alpha,
-                                             CoglTextureFlags flags);
 CoglPipeline * meta_create_texture_pipeline (CoglTexture *texture);
 
 #endif /* __META_COGL_UTILS_H__ */
