@@ -32,7 +32,10 @@
 
 #include "meta-surface-actor.h"
 #include "meta-surface-actor-x11.h"
+
+#ifdef HAVE_WAYLAND
 #include "meta-surface-actor-wayland.h"
+#endif
 
 #include "wayland/meta-wayland-surface.h"
 
@@ -76,7 +79,7 @@ struct _MetaWindowActorPrivate
   gint64            frame_drawn_time;
 
   guint             repaint_scheduled_id;
-  guint             allocation_changed_id;
+  guint             size_changed_id;
 
   /*
    * These need to be counters rather than flags, since more plugins
@@ -84,6 +87,7 @@ struct _MetaWindowActorPrivate
    * might be dubious, but we have to at least handle it correctly.
    */
   gint              minimize_in_progress;
+  gint              unminimize_in_progress;
   gint              maximize_in_progress;
   gint              unmaximize_in_progress;
   gint              map_in_progress;
@@ -272,12 +276,11 @@ window_appears_focused_notify (MetaWindow *mw,
 }
 
 static void
-surface_allocation_changed_notify (ClutterActor           *actor,
-                                   const ClutterActorBox  *allocation,
-                                   ClutterAllocationFlags  flags,
-                                   MetaWindowActor        *self)
+surface_size_changed (MetaSurfaceActor *actor,
+                      gpointer          user_data)
 {
-  meta_window_actor_sync_actor_geometry (self, FALSE);
+  MetaWindowActor *self = META_WINDOW_ACTOR (user_data);
+
   meta_window_actor_update_shape (self);
 }
 
@@ -369,9 +372,8 @@ set_surface (MetaWindowActor  *self,
   if (priv->surface)
     {
       g_signal_handler_disconnect (priv->surface, priv->repaint_scheduled_id);
+      g_signal_handler_disconnect (priv->surface, priv->size_changed_id);
       priv->repaint_scheduled_id = 0;
-      g_signal_handler_disconnect (priv->surface, priv->allocation_changed_id);
-      priv->allocation_changed_id = 0;
       clutter_actor_remove_child (CLUTTER_ACTOR (self), CLUTTER_ACTOR (priv->surface));
       g_object_unref (priv->surface);
     }
@@ -383,8 +385,8 @@ set_surface (MetaWindowActor  *self,
       g_object_ref_sink (priv->surface);
       priv->repaint_scheduled_id = g_signal_connect (priv->surface, "repaint-scheduled",
                                                      G_CALLBACK (surface_repaint_scheduled), self);
-      priv->allocation_changed_id = g_signal_connect (priv->surface, "allocation-changed",
-                                                      G_CALLBACK (surface_allocation_changed_notify), self);
+      priv->size_changed_id = g_signal_connect (priv->surface, "size-changed",
+                                                G_CALLBACK (surface_size_changed), self);
       clutter_actor_add_child (CLUTTER_ACTOR (self), CLUTTER_ACTOR (priv->surface));
 
       /* If the previous surface actor was frozen, start out
@@ -597,6 +599,7 @@ meta_window_actor_get_shape_bounds (MetaWindowActor       *self,
 
   cairo_region_get_extents (priv->shape_region, bounds);
 
+#ifdef HAVE_WAYLAND
   if (META_IS_SURFACE_ACTOR_WAYLAND (priv->surface))
     {
       double scale = priv->surface ?
@@ -606,6 +609,7 @@ meta_window_actor_get_shape_bounds (MetaWindowActor       *self,
       bounds->width *= scale;
       bounds->height *= scale;
     }
+#endif
 }
 
 static void
@@ -713,25 +717,16 @@ meta_window_actor_get_paint_volume (ClutterActor       *actor,
 {
   MetaWindowActor *self = META_WINDOW_ACTOR (actor);
   MetaWindowActorPrivate *priv = self->priv;
-  cairo_rectangle_int_t unobscured_bounds, bounds;
   gboolean appears_focused = meta_window_appears_focused (priv->window);
-  ClutterVertex origin;
 
   /* The paint volume is computed before paint functions are called
    * so our bounds might not be updated yet. Force an update. */
   meta_window_actor_handle_updates (self);
 
-  meta_window_actor_get_shape_bounds (self, &bounds);
-
-  if (priv->surface)
-    {
-      if (meta_surface_actor_get_unobscured_bounds (priv->surface, &unobscured_bounds))
-        gdk_rectangle_intersect (&bounds, &unobscured_bounds, &bounds);
-    }
-
   if (appears_focused ? priv->focused_shadow : priv->unfocused_shadow)
     {
       cairo_rectangle_int_t shadow_bounds;
+      ClutterActorBox shadow_box;
 
       /* We could compute an full clip region as we do for the window
        * texture, but the shadow is relatively cheap to draw, and
@@ -741,16 +736,24 @@ meta_window_actor_get_paint_volume (ClutterActor       *actor,
        */
 
       meta_window_actor_get_shadow_bounds (self, appears_focused, &shadow_bounds);
-      gdk_rectangle_union (&bounds, &shadow_bounds, &bounds);
+      shadow_box.x1 = shadow_bounds.x;
+      shadow_box.x2 = shadow_bounds.x + shadow_bounds.width;
+      shadow_box.y1 = shadow_bounds.y;
+      shadow_box.y2 = shadow_bounds.y + shadow_bounds.height;
+
+      clutter_paint_volume_union_box (volume, &shadow_box);
     }
 
-  origin.x = bounds.x;
-  origin.y = bounds.y;
-  origin.z = 0.0f;
-  clutter_paint_volume_set_origin (volume, &origin);
+  if (priv->surface)
+    {
+      const ClutterPaintVolume *child_volume;
 
-  clutter_paint_volume_set_width (volume, bounds.width);
-  clutter_paint_volume_set_height (volume, bounds.height);
+      child_volume = clutter_actor_get_transformed_paint_volume (CLUTTER_ACTOR (priv->surface), actor);
+      if (!child_volume)
+        return FALSE;
+
+      clutter_paint_volume_union (volume, child_volume);
+    }
 
   return TRUE;
 }
@@ -1008,6 +1011,9 @@ start_simple_effect (MetaWindowActor *self,
   case META_PLUGIN_MINIMIZE:
     counter = &priv->minimize_in_progress;
     break;
+  case META_PLUGIN_UNMINIMIZE:
+    counter = &priv->unminimize_in_progress;
+    break;
   case META_PLUGIN_MAP:
     counter = &priv->map_in_progress;
     break;
@@ -1078,6 +1084,16 @@ meta_window_actor_effect_completed (MetaWindowActor *self,
 	}
     }
     break;
+  case META_PLUGIN_UNMINIMIZE:
+    {
+      priv->unminimize_in_progress--;
+      if (priv->unminimize_in_progress < 0)
+       {
+         g_warning ("Error in unminimize accounting.");
+         priv->unminimize_in_progress = 0;
+       }
+    }
+    break;
   case META_PLUGIN_MAP:
     /*
      * Make sure that the actor is at the correct place in case
@@ -1243,8 +1259,7 @@ meta_window_actor_show (MetaWindowActor   *self,
       event = META_PLUGIN_MAP;
       break;
     case META_COMP_EFFECT_UNMINIMIZE:
-      /* FIXME: should have META_PLUGIN_UNMINIMIZE */
-      event = META_PLUGIN_MAP;
+      event = META_PLUGIN_UNMINIMIZE;
       break;
     case META_COMP_EFFECT_NONE:
       break;
@@ -1647,6 +1662,8 @@ build_and_scan_frame_mask (MetaWindowActor       *self,
                            cairo_rectangle_int_t *client_area,
                            cairo_region_t        *shape_region)
 {
+  ClutterBackend *backend = clutter_get_default_backend ();
+  CoglContext *ctx = clutter_backend_get_cogl_context (backend);
   MetaWindowActorPrivate *priv = self->priv;
   guchar *mask_data;
   guint tex_width, tex_height;
@@ -1709,10 +1726,7 @@ build_and_scan_frame_mask (MetaWindowActor       *self,
 
   if (meta_texture_rectangle_check (paint_tex))
     {
-      ClutterBackend *backend = clutter_get_default_backend ();
-      CoglContext *context = clutter_backend_get_cogl_context (backend);
-
-      mask_texture = COGL_TEXTURE (cogl_texture_rectangle_new_with_size (context, tex_width, tex_height));
+      mask_texture = COGL_TEXTURE (cogl_texture_rectangle_new_with_size (ctx, tex_width, tex_height));
       cogl_texture_set_components (mask_texture, COGL_TEXTURE_COMPONENTS_A);
       cogl_texture_set_region (mask_texture,
                                0, 0, /* src_x/y */
@@ -1724,15 +1738,9 @@ build_and_scan_frame_mask (MetaWindowActor       *self,
     }
   else
     {
-      /* Note: we don't allow slicing for this texture because we
-       * need to use it with multi-texturing which doesn't support
-       * sliced textures */
-      mask_texture = cogl_texture_new_from_data (tex_width, tex_height,
-                                                 COGL_TEXTURE_NO_SLICING,
-                                                 COGL_PIXEL_FORMAT_A_8,
-                                                 COGL_PIXEL_FORMAT_ANY,
-                                                 stride,
-                                                 mask_data);
+      mask_texture = COGL_TEXTURE (cogl_texture_2d_new_from_data (ctx, tex_width, tex_height,
+                                                                  COGL_PIXEL_FORMAT_A_8,
+                                                                  stride, mask_data, NULL));
     }
 
   meta_shaped_texture_set_mask_texture (stex, mask_texture);
