@@ -49,7 +49,7 @@
 #include "mutter-enum-types.h"
 #include "meta-idle-monitor-dbus.h"
 #include "meta-cursor-tracker-private.h"
-#include "meta-backend.h"
+#include <meta/meta-backend.h>
 #include "backends/x11/meta-backend-x11.h"
 #include <clutter/x11/clutter-x11.h>
 
@@ -65,12 +65,15 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "x11/events.h"
 #include "x11/window-x11.h"
 #include "x11/window-props.h"
 #include "x11/group-props.h"
 #include "x11/xprops.h"
 
+#ifdef HAVE_WAYLAND
 #include "wayland/meta-xwayland-private.h"
+#endif
 
 /*
  * SECTION:pings
@@ -143,7 +146,6 @@ static const char *gnome_wm_keybindings = "Mutter";
 static const char *net_wm_name = "Mutter";
 
 static void update_cursor_theme (void);
-static void    update_window_grab_modifiers (MetaDisplay *display);
 
 static void    prefs_changed_callback    (MetaPreference pref,
                                           void          *data);
@@ -415,11 +417,15 @@ enable_compositor (MetaDisplay *display)
       return;
     }
 
-  if (!display->compositor)
-      display->compositor = meta_compositor_new (display);
+  int version = (display->composite_major_version * 10) + display->composite_minor_version;
+  if (version < 3)
+    {
+      meta_warning ("Your version of COMPOSITE is too old.");
+      return;
+    }
 
   if (!display->compositor)
-    return;
+      display->compositor = meta_compositor_new (display);
 
   meta_compositor_manage (display->compositor);
 }
@@ -461,26 +467,48 @@ meta_set_gnome_wm_keybindings (const char *wm_keybindings)
   gnome_wm_keybindings = wm_keybindings;
 }
 
+void
+meta_display_cancel_touch (MetaDisplay *display)
+{
+#ifdef HAVE_WAYLAND
+  MetaWaylandCompositor *compositor;
+
+  if (!meta_is_wayland_compositor ())
+    return;
+
+  compositor = meta_wayland_compositor_get_default ();
+  meta_wayland_touch_cancel (&compositor->seat->touch);
+#endif
+}
+
 static void
 gesture_tracker_state_changed (MetaGestureTracker   *tracker,
                                ClutterEventSequence *sequence,
                                MetaSequenceState     state,
                                MetaDisplay          *display)
 {
-  MetaBackendX11 *backend = META_BACKEND_X11 (meta_get_backend ());
-  int event_mode;
-
-  if (state == META_SEQUENCE_ACCEPTED)
-    event_mode = XIAcceptTouch;
-  else if (state == META_SEQUENCE_REJECTED)
-    event_mode = XIRejectTouch;
+  if (meta_is_wayland_compositor ())
+    {
+      if (state == META_SEQUENCE_ACCEPTED)
+        meta_display_cancel_touch (display);
+    }
   else
-    return;
+    {
+      MetaBackendX11 *backend = META_BACKEND_X11 (meta_get_backend ());
+      int event_mode;
 
-  XIAllowTouchEvents (meta_backend_x11_get_xdisplay (backend),
-                      META_VIRTUAL_CORE_POINTER_ID,
-                      clutter_x11_event_sequence_get_touch_detail (sequence),
-                      DefaultRootWindow (display->xdisplay), event_mode);
+      if (state == META_SEQUENCE_ACCEPTED)
+        event_mode = XIAcceptTouch;
+      else if (state == META_SEQUENCE_REJECTED)
+        event_mode = XIRejectTouch;
+      else
+        return;
+
+      XIAllowTouchEvents (meta_backend_x11_get_xdisplay (backend),
+                          META_VIRTUAL_CORE_POINTER_ID,
+                          clutter_x11_event_sequence_get_touch_detail (sequence),
+                          DefaultRootWindow (display->xdisplay), event_mode);
+    }
 }
 
 /**
@@ -521,8 +549,10 @@ meta_display_open (void)
       return FALSE;
     }
 
+#ifdef HAVE_WAYLAND
   if (meta_is_wayland_compositor ())
     meta_xwayland_complete_init ();
+#endif
 
   if (meta_is_syncing ())
     XSynchronize (xdisplay, True);
@@ -556,8 +586,6 @@ meta_display_open (void)
 
   meta_display_init_keys (display);
 
-  update_window_grab_modifiers (display);
-
   meta_prefs_add_listener (prefs_changed_callback, display);
 
   meta_verbose ("Creating %d atoms\n", (int) G_N_ELEMENTS (atom_names));
@@ -595,6 +623,7 @@ meta_display_open (void)
 
   /* Get events */
   meta_display_init_events (display);
+  meta_display_init_events_x11 (display);
 
   display->xids = g_hash_table_new (meta_unsigned_long_hash,
                                         meta_unsigned_long_equal);
@@ -614,9 +643,7 @@ meta_display_open (void)
   display->grab_resize_timeout_id = 0;
   display->grab_have_keyboard = FALSE;
 
-#ifdef HAVE_XKB
   display->last_bell_time = 0;
-#endif
 
   display->grab_op = META_GRAB_OP_NONE;
   display->grab_window = NULL;
@@ -1061,6 +1088,7 @@ meta_display_close (MetaDisplay *display,
   display->focus_timeout_id = 0;
 
   /* Stop caring about events */
+  meta_display_free_events_x11 (display);
   meta_display_free_events (display);
 
   meta_screen_free (display->screen, timestamp);
@@ -1139,117 +1167,64 @@ meta_get_display (void)
   return the_display;
 }
 
+static inline gboolean
+grab_op_is_window (MetaGrabOp op)
+{
+  return GRAB_OP_GET_BASE_TYPE (op) == META_GRAB_OP_WINDOW_BASE;
+}
+
 gboolean
 meta_grab_op_is_mouse (MetaGrabOp op)
 {
-  switch (op)
-    {
-    case META_GRAB_OP_MOVING:
-    case META_GRAB_OP_RESIZING_SE:
-    case META_GRAB_OP_RESIZING_S:
-    case META_GRAB_OP_RESIZING_SW:
-    case META_GRAB_OP_RESIZING_N:
-    case META_GRAB_OP_RESIZING_NE:
-    case META_GRAB_OP_RESIZING_NW:
-    case META_GRAB_OP_RESIZING_W:
-    case META_GRAB_OP_RESIZING_E:
-      return TRUE;
+  if (!grab_op_is_window (op))
+    return FALSE;
 
-    default:
-      return FALSE;
-    }
+  return (op & META_GRAB_OP_WINDOW_FLAG_KEYBOARD) == 0;
 }
 
 gboolean
 meta_grab_op_is_keyboard (MetaGrabOp op)
 {
-  switch (op)
-    {
-    case META_GRAB_OP_KEYBOARD_MOVING:
-    case META_GRAB_OP_KEYBOARD_RESIZING_UNKNOWN:
-    case META_GRAB_OP_KEYBOARD_RESIZING_S:
-    case META_GRAB_OP_KEYBOARD_RESIZING_N:
-    case META_GRAB_OP_KEYBOARD_RESIZING_W:
-    case META_GRAB_OP_KEYBOARD_RESIZING_E:
-    case META_GRAB_OP_KEYBOARD_RESIZING_SE:
-    case META_GRAB_OP_KEYBOARD_RESIZING_NE:
-    case META_GRAB_OP_KEYBOARD_RESIZING_SW:
-    case META_GRAB_OP_KEYBOARD_RESIZING_NW:
-      return TRUE;
+  if (!grab_op_is_window (op))
+    return FALSE;
 
-    default:
-      return FALSE;
-    }
+  return (op & META_GRAB_OP_WINDOW_FLAG_KEYBOARD) != 0;
 }
 
 gboolean
 meta_grab_op_is_resizing (MetaGrabOp op)
 {
-  switch (op)
-    {
-    case META_GRAB_OP_RESIZING_SE:
-    case META_GRAB_OP_RESIZING_S:
-    case META_GRAB_OP_RESIZING_SW:
-    case META_GRAB_OP_RESIZING_N:
-    case META_GRAB_OP_RESIZING_NE:
-    case META_GRAB_OP_RESIZING_NW:
-    case META_GRAB_OP_RESIZING_W:
-    case META_GRAB_OP_RESIZING_E:
-    case META_GRAB_OP_KEYBOARD_RESIZING_UNKNOWN:
-    case META_GRAB_OP_KEYBOARD_RESIZING_S:
-    case META_GRAB_OP_KEYBOARD_RESIZING_N:
-    case META_GRAB_OP_KEYBOARD_RESIZING_W:
-    case META_GRAB_OP_KEYBOARD_RESIZING_E:
-    case META_GRAB_OP_KEYBOARD_RESIZING_SE:
-    case META_GRAB_OP_KEYBOARD_RESIZING_NE:
-    case META_GRAB_OP_KEYBOARD_RESIZING_SW:
-    case META_GRAB_OP_KEYBOARD_RESIZING_NW:
-      return TRUE;
+  if (!grab_op_is_window (op))
+    return FALSE;
 
-    default:
-      return FALSE;
-    }
+  return (op & META_GRAB_OP_WINDOW_DIR_MASK) != 0;
 }
 
 gboolean
 meta_grab_op_is_moving (MetaGrabOp op)
 {
-  switch (op)
-    {
-    case META_GRAB_OP_MOVING:
-    case META_GRAB_OP_KEYBOARD_MOVING:
-      return TRUE;
+  if (!grab_op_is_window (op))
+    return FALSE;
 
-    default:
-      return FALSE;
-    }
-}
-
-gboolean
-meta_grab_op_is_moving_or_resizing (MetaGrabOp op)
-{
-  return (meta_grab_op_is_moving (op) ||
-          meta_grab_op_is_resizing (op));
+  return (op & META_GRAB_OP_WINDOW_DIR_MASK) == 0;
 }
 
 /**
- * meta_grab_op_should_block_wayland:
+ * meta_grab_op_windows_are_interactable:
  * @op: A #MetaGrabOp
  *
- * Starting a grab with one of these grab operations means
- * that we will remove key / pointer focus from the current
- * Wayland focus.
+ * Whether windows can be interacted with.
  */
 gboolean
-meta_grab_op_should_block_wayland (MetaGrabOp op)
+meta_display_windows_are_interactable (MetaDisplay *display)
 {
-  switch (op)
+  switch (display->event_route)
     {
-    case META_GRAB_OP_WAYLAND_POPUP:
-    case META_GRAB_OP_NONE:
-      return FALSE;
-    default:
+    case META_EVENT_ROUTE_NORMAL:
+    case META_EVENT_ROUTE_WAYLAND_POPUP:
       return TRUE;
+    default:
+      return FALSE;
     }
 }
 
@@ -1414,10 +1389,11 @@ meta_display_queue_autoraise_callback (MetaDisplay *display,
 void
 meta_display_sync_wayland_input_focus (MetaDisplay *display)
 {
+#ifdef HAVE_WAYLAND
   MetaWaylandCompositor *compositor = meta_wayland_compositor_get_default ();
   MetaWindow *focus_window = NULL;
 
-  if (meta_grab_op_should_block_wayland (display->grab_op))
+  if (!meta_display_windows_are_interactable (display))
     focus_window = NULL;
   else if (meta_display_xwindow_is_a_no_focus_window (display, display->focus_xwindow))
     focus_window = NULL;
@@ -1429,6 +1405,7 @@ meta_display_sync_wayland_input_focus (MetaDisplay *display)
   meta_wayland_compositor_set_input_focus (compositor, focus_window);
 
   meta_wayland_seat_repick (compositor->seat);
+#endif
 }
 
 void
@@ -1725,6 +1702,30 @@ get_first_freefloating_window (MetaWindow *window)
   return window;
 }
 
+static MetaEventRoute
+get_event_route_from_grab_op (MetaGrabOp op)
+{
+  switch (GRAB_OP_GET_BASE_TYPE (op))
+    {
+    case META_GRAB_OP_NONE:
+      /* begin_grab_op shouldn't be called with META_GRAB_OP_NONE. */
+      g_assert_not_reached ();
+
+    case META_GRAB_OP_WINDOW_BASE:
+      return META_EVENT_ROUTE_WINDOW_OP;
+
+    case META_GRAB_OP_COMPOSITOR:
+      /* begin_grab_op shouldn't be called with META_GRAB_OP_COMPOSITOR. */
+      g_assert_not_reached ();
+
+    case META_GRAB_OP_WAYLAND_POPUP:
+      return META_EVENT_ROUTE_WAYLAND_POPUP;
+
+    default:
+      g_assert_not_reached ();
+    }
+}
+
 gboolean
 meta_display_begin_grab_op (MetaDisplay *display,
 			    MetaScreen  *screen,
@@ -1740,6 +1741,7 @@ meta_display_begin_grab_op (MetaDisplay *display,
 {
   MetaBackend *backend = meta_get_backend ();
   MetaWindow *grab_window = NULL;
+  MetaEventRoute event_route;
 
   g_assert (window != NULL);
 
@@ -1756,7 +1758,9 @@ meta_display_begin_grab_op (MetaDisplay *display,
       return FALSE;
     }
 
-  if (meta_grab_op_is_moving (op) || meta_grab_op_is_resizing (op))
+  event_route = get_event_route_from_grab_op (op);
+
+  if (event_route == META_EVENT_ROUTE_WINDOW_OP)
     {
       if (meta_prefs_get_raise_on_click ())
         meta_window_raise (window);
@@ -1802,8 +1806,8 @@ meta_display_begin_grab_op (MetaDisplay *display,
       return FALSE;
     }
 
-  /* Grab keys for keyboard ops and mouse move/resizes; see #126497 */
-  if (meta_grab_op_is_moving_or_resizing (op))
+  /* Grab keys when beginning window ops; see #126497 */
+  if (event_route == META_EVENT_ROUTE_WINDOW_OP)
     {
       display->grab_have_keyboard = meta_window_grab_all_keys (grab_window, timestamp);
 
@@ -1816,6 +1820,7 @@ meta_display_begin_grab_op (MetaDisplay *display,
         }
     }
 
+  display->event_route = event_route;
   display->grab_op = op;
   display->grab_window = grab_window;
   display->grab_button = button;
@@ -1830,7 +1835,6 @@ meta_display_begin_grab_op (MetaDisplay *display,
   display->grab_last_user_action_was_snap = FALSE;
   display->grab_frame_action = frame_action;
   display->grab_resize_unmaximize = 0;
-  display->grab_timestamp = timestamp;
 
   meta_display_update_cursor (display);
 
@@ -1849,12 +1853,16 @@ meta_display_begin_grab_op (MetaDisplay *display,
   display->grab_anchor_window_pos = display->grab_initial_window_pos;
 
   if (meta_is_wayland_compositor ())
-    meta_display_sync_wayland_input_focus (display);
+    {
+      meta_display_sync_wayland_input_focus (display);
+      meta_display_cancel_touch (display);
+    }
 
   g_signal_emit (display, display_signals[GRAB_OP_BEGIN], 0,
                  screen, display->grab_window, display->grab_op);
 
-  meta_window_grab_op_began (display->grab_window, display->grab_op);
+  if (display->event_route == META_EVENT_ROUTE_WINDOW_OP)
+    meta_window_grab_op_began (display->grab_window, display->grab_op);
 
   return TRUE;
 }
@@ -1869,13 +1877,13 @@ meta_display_end_grab_op (MetaDisplay *display,
   meta_topic (META_DEBUG_WINDOW_OPS,
               "Ending grab op %u at time %u\n", grab_op, timestamp);
 
-  if (display->grab_op == META_GRAB_OP_NONE)
+  if (display->event_route == META_EVENT_ROUTE_NORMAL)
     return;
 
   g_signal_emit (display, display_signals[GRAB_OP_END], 0,
                  display->screen, grab_window, grab_op);
 
-  if (meta_grab_op_is_moving_or_resizing (grab_op))
+  if (display->event_route == META_EVENT_ROUTE_WINDOW_OP)
     {
       /* Clear out the edge cache */
       meta_display_cleanup_edges (display);
@@ -1889,6 +1897,8 @@ meta_display_end_grab_op (MetaDisplay *display,
       if (!meta_prefs_get_raise_on_click () &&
           display->grab_threshold_movement_reached)
         meta_window_raise (display->grab_window);
+
+      meta_window_grab_op_ended (grab_window, grab_op);
     }
 
   if (display->grab_have_pointer)
@@ -1904,11 +1914,11 @@ meta_display_end_grab_op (MetaDisplay *display,
       meta_window_ungrab_all_keys (grab_window, timestamp);
     }
 
-  display->grab_timestamp = 0;
+  display->event_route = META_EVENT_ROUTE_NORMAL;
+  display->grab_op = META_GRAB_OP_NONE;
   display->grab_window = NULL;
   display->grab_tile_mode = META_TILE_NONE;
   display->grab_tile_monitor_number = -1;
-  display->grab_op = META_GRAB_OP_NONE;
 
   meta_display_update_cursor (display);
 
@@ -1918,8 +1928,6 @@ meta_display_end_grab_op (MetaDisplay *display,
       display->grab_resize_timeout_id = 0;
     }
 
-  meta_window_grab_op_ended (grab_window, grab_op);
-
   if (meta_is_wayland_compositor ())
     meta_display_sync_wayland_input_focus (display);
 }
@@ -1956,221 +1964,6 @@ meta_display_check_threshold_reached (MetaDisplay *display,
     display->grab_threshold_movement_reached = TRUE;
 }
 
-static void
-meta_change_button_grab (MetaDisplay *display,
-                         Window       xwindow,
-                         gboolean     grab,
-                         gboolean     sync,
-                         int          button,
-                         int          modmask)
-{
-  MetaBackendX11 *backend = META_BACKEND_X11 (meta_get_backend ());
-  Display *xdisplay = meta_backend_x11_get_xdisplay (backend);
-
-  unsigned int ignored_mask;
-  unsigned char mask_bits[XIMaskLen (XI_LASTEVENT)] = { 0 };
-  XIEventMask mask = { XIAllMasterDevices, sizeof (mask_bits), mask_bits };
-
-  XISetMask (mask.mask, XI_ButtonPress);
-  XISetMask (mask.mask, XI_ButtonRelease);
-  XISetMask (mask.mask, XI_Motion);
-
-  ignored_mask = 0;
-  while (ignored_mask <= display->ignored_modifier_mask)
-    {
-      XIGrabModifiers mods;
-
-      if (ignored_mask & ~(display->ignored_modifier_mask))
-        {
-          /* Not a combination of ignored modifiers
-           * (it contains some non-ignored modifiers)
-           */
-          ++ignored_mask;
-          continue;
-        }
-
-      mods = (XIGrabModifiers) { modmask | ignored_mask, 0 };
-
-      /* GrabModeSync means freeze until XAllowEvents */
-
-      if (grab)
-        XIGrabButton (xdisplay,
-                      META_VIRTUAL_CORE_POINTER_ID,
-                      button, xwindow, None,
-                      sync ? XIGrabModeSync : XIGrabModeAsync,
-                      XIGrabModeAsync, False,
-                      &mask, 1, &mods);
-      else
-        XIUngrabButton (xdisplay,
-                        META_VIRTUAL_CORE_POINTER_ID,
-                        button, xwindow, 1, &mods);
-
-      ++ignored_mask;
-    }
-}
-
-void
-meta_display_grab_window_buttons (MetaDisplay *display,
-                                  Window       xwindow)
-{
-  if (meta_is_wayland_compositor ())
-    return;
-
-  /* Grab Alt + button1 for moving window.
-   * Grab Alt + button2 for resizing window.
-   * Grab Alt + button3 for popping up window menu.
-   * Grab Alt + Shift + button1 for snap-moving window.
-   */
-  meta_verbose ("Grabbing window buttons for 0x%lx\n", xwindow);
-
-  /* FIXME If we ignored errors here instead of spewing, we could
-   * put one big error trap around the loop and avoid a bunch of
-   * XSync()
-   */
-
-  if (display->window_grab_modifiers != 0)
-    {
-      gboolean debug = g_getenv ("MUTTER_DEBUG_BUTTON_GRABS") != NULL;
-      int i;
-      for (i = 1; i < 4; i++)
-        {
-          meta_change_button_grab (display, xwindow,
-                                   TRUE,
-                                   FALSE,
-                                   i, display->window_grab_modifiers);
-
-          /* This is for debugging, since I end up moving the Xnest
-           * otherwise ;-)
-           */
-          if (debug)
-            meta_change_button_grab (display, xwindow,
-                                     TRUE,
-                                     FALSE,
-                                     i, ControlMask);
-        }
-
-      /* In addition to grabbing Alt+Button1 for moving the window,
-       * grab Alt+Shift+Button1 for snap-moving the window.  See bug
-       * 112478.  Unfortunately, this doesn't work with
-       * Shift+Alt+Button1 for some reason; so at least part of the
-       * order still matters, which sucks (please FIXME).
-       */
-      meta_change_button_grab (display, xwindow,
-                               TRUE,
-                               FALSE,
-                               1, display->window_grab_modifiers | ShiftMask);
-    }
-}
-
-void
-meta_display_ungrab_window_buttons  (MetaDisplay *display,
-                                     Window       xwindow)
-{
-  gboolean debug;
-  int i;
-
-  if (meta_is_wayland_compositor ())
-    return;
-
-  if (display->window_grab_modifiers == 0)
-    return;
-
-  debug = g_getenv ("MUTTER_DEBUG_BUTTON_GRABS") != NULL;
-  i = 1;
-  while (i < 4)
-    {
-      meta_change_button_grab (display, xwindow,
-                               FALSE, FALSE, i,
-                               display->window_grab_modifiers);
-
-      if (debug)
-        meta_change_button_grab (display, xwindow,
-                                 FALSE, FALSE, i, ControlMask);
-
-      ++i;
-    }
-}
-
-/* Grab buttons we only grab while unfocused in click-to-focus mode */
-#define MAX_FOCUS_BUTTON 4
-void
-meta_display_grab_focus_window_button (MetaDisplay *display,
-                                       MetaWindow  *window)
-{
-  if (meta_is_wayland_compositor ())
-    return;
-
-  /* Grab button 1 for activating unfocused windows */
-  meta_verbose ("Grabbing unfocused window buttons for %s\n", window->desc);
-
-#if 0
-  /* FIXME:115072 */
-  /* Don't grab at all unless in click to focus mode. In click to
-   * focus, we may sometimes be clever about intercepting and eating
-   * the focus click. But in mouse focus, we never do that since the
-   * focus window may not be raised, and who wants to think about
-   * mouse focus anyway.
-   */
-  if (meta_prefs_get_focus_mode () != G_DESKTOP_FOCUS_MODE_CLICK)
-    {
-      meta_verbose (" (well, not grabbing since not in click to focus mode)\n");
-      return;
-    }
-#endif
-
-  if (window->have_focus_click_grab)
-    {
-      meta_verbose (" (well, not grabbing since we already have the grab)\n");
-      return;
-    }
-
-  /* FIXME If we ignored errors here instead of spewing, we could
-   * put one big error trap around the loop and avoid a bunch of
-   * XSync()
-   */
-
-  {
-    int i = 1;
-    while (i < MAX_FOCUS_BUTTON)
-      {
-        meta_change_button_grab (display,
-                                 window->xwindow,
-                                 TRUE, TRUE,
-                                 i, 0);
-
-        ++i;
-      }
-
-    window->have_focus_click_grab = TRUE;
-  }
-}
-
-void
-meta_display_ungrab_focus_window_button (MetaDisplay *display,
-                                         MetaWindow  *window)
-{
-  if (meta_is_wayland_compositor ())
-    return;
-
-  meta_verbose ("Ungrabbing unfocused window buttons for %s\n", window->desc);
-
-  if (!window->have_focus_click_grab)
-    return;
-
-  {
-    int i = 1;
-    while (i < MAX_FOCUS_BUTTON)
-      {
-        meta_change_button_grab (display, window->xwindow,
-                                 FALSE, FALSE, i, 0);
-
-        ++i;
-      }
-
-    window->have_focus_click_grab = FALSE;
-  }
-}
-
 void
 meta_display_increment_event_serial (MetaDisplay *display)
 {
@@ -2820,92 +2613,23 @@ meta_display_sort_windows_by_stacking (MetaDisplay *display,
   return copy;
 }
 
-void
-meta_display_devirtualize_modifiers (MetaDisplay        *display,
-                                     MetaVirtualModifier modifiers,
-                                     unsigned int       *mask)
-{
-  *mask = 0;
-
-  if (modifiers & META_VIRTUAL_SHIFT_MASK)
-    *mask |= ShiftMask;
-  if (modifiers & META_VIRTUAL_CONTROL_MASK)
-    *mask |= ControlMask;
-  if (modifiers & META_VIRTUAL_ALT_MASK)
-    *mask |= Mod1Mask;
-  if (modifiers & META_VIRTUAL_META_MASK)
-    *mask |= display->meta_mask;
-  if (modifiers & META_VIRTUAL_HYPER_MASK)
-    *mask |= display->hyper_mask;
-  if (modifiers & META_VIRTUAL_SUPER_MASK)
-    *mask |= display->super_mask;
-  if (modifiers & META_VIRTUAL_MOD2_MASK)
-    *mask |= Mod2Mask;
-  if (modifiers & META_VIRTUAL_MOD3_MASK)
-    *mask |= Mod3Mask;
-  if (modifiers & META_VIRTUAL_MOD4_MASK)
-    *mask |= Mod4Mask;
-  if (modifiers & META_VIRTUAL_MOD5_MASK)
-    *mask |= Mod5Mask;
-}
-
-static void
-update_window_grab_modifiers (MetaDisplay *display)
-{
-  MetaVirtualModifier virtual_mods;
-  unsigned int mods;
-
-  virtual_mods = meta_prefs_get_mouse_button_mods ();
-  meta_display_devirtualize_modifiers (display, virtual_mods,
-                                       &mods);
-
-  display->window_grab_modifiers = mods;
-}
-
 static void
 prefs_changed_callback (MetaPreference pref,
                         void          *data)
 {
   MetaDisplay *display = data;
 
-  /* It may not be obvious why we regrab on focus mode
-   * change; it's because we handle focus clicks a
-   * bit differently for the different focus modes.
-   */
-  if (pref == META_PREF_MOUSE_BUTTON_MODS ||
-      pref == META_PREF_FOCUS_MODE)
+  if (pref == META_PREF_FOCUS_MODE)
     {
-      MetaDisplay *display = data;
-      GSList *windows;
-      GSList *tmp;
-
+      GSList *windows, *l;
       windows = meta_display_list_windows (display, META_LIST_DEFAULT);
 
-      /* Ungrab all */
-      tmp = windows;
-      while (tmp != NULL)
+      for (l = windows; l; l = l->next)
         {
-          MetaWindow *w = tmp->data;
-          meta_display_ungrab_window_buttons (display, w->xwindow);
+          MetaWindow *w = l->data;
           meta_display_ungrab_focus_window_button (display, w);
-          tmp = tmp->next;
-        }
-
-      /* change our modifier */
-      if (pref == META_PREF_MOUSE_BUTTON_MODS)
-        update_window_grab_modifiers (display);
-
-      /* Grab all */
-      tmp = windows;
-      while (tmp != NULL)
-        {
-          MetaWindow *w = tmp->data;
           if (w->type != META_WINDOW_DOCK)
-            {
-              meta_display_grab_focus_window_button (display, w);
-              meta_display_grab_window_buttons (display, w->xwindow);
-            }
-          tmp = tmp->next;
+            meta_display_grab_focus_window_button (display, w);
         }
 
       g_slist_free (windows);
@@ -3074,15 +2798,6 @@ meta_display_modifiers_accelerator_activate (MetaDisplay *display)
   return freeze;
 }
 
-void
-meta_display_get_compositor_version (MetaDisplay *display,
-                                     int         *major,
-                                     int         *minor)
-{
-  *major = display->composite_major_version;
-  *minor = display->composite_minor_version;
-}
-
 /**
  * meta_display_get_xinput_opcode: (skip)
  * @display: a #MetaDisplay
@@ -3219,3 +2934,23 @@ meta_display_request_restart (MetaDisplay *display)
 
   return result;
 }
+
+/**
+ * meta_display_is_pointer_emulating_sequence:
+ * @display: the display
+ * @sequence: (nullable): a #ClutterEventSequence
+ *
+ * Tells whether the event sequence is the used for pointer emulation
+ * and single-touch interaction.
+ *
+ * Returns: #TRUE if the sequence emulates pointer behavior
+ **/
+gboolean
+meta_display_is_pointer_emulating_sequence (MetaDisplay          *display,
+                                            ClutterEventSequence *sequence)
+{
+  if (!sequence)
+    return FALSE;
+
+  return display->pointer_emulating_sequence == sequence;
+}
