@@ -23,9 +23,10 @@
 #include "config.h"
 #include "events.h"
 
+#include <meta/meta-backend.h>
+
 #include "display-private.h"
 #include "window-private.h"
-#include "backends/meta-backend.h"
 #include "backends/x11/meta-backend-x11.h"
 
 #ifdef HAVE_NATIVE_BACKEND
@@ -33,34 +34,44 @@
 #include "backends/native/meta-idle-monitor-native.h"
 #endif
 
-#include "x11/events.h"
+#ifdef HAVE_WAYLAND
 #include "wayland/meta-wayland-private.h"
+#endif
 #include "meta-surface-actor.h"
 
 static MetaWindow *
 get_window_for_event (MetaDisplay        *display,
                       const ClutterEvent *event)
 {
-  ClutterActor *source;
-
-  if (display->grab_op != META_GRAB_OP_NONE)
-    return display->grab_window;
-
-  /* Always use the key focused window for key events. */
-  switch (event->type)
+  switch (display->event_route)
     {
-    case CLUTTER_KEY_PRESS:
-    case CLUTTER_KEY_RELEASE:
-      return display->focus_window;
+    case META_EVENT_ROUTE_NORMAL:
+      {
+        ClutterActor *source;
+
+        /* Always use the key focused window for key events. */
+        switch (event->type)
+          {
+          case CLUTTER_KEY_PRESS:
+          case CLUTTER_KEY_RELEASE:
+            return display->focus_window;
+          default:
+            break;
+          }
+
+        source = clutter_event_get_source (event);
+        if (META_IS_SURFACE_ACTOR (source))
+          return meta_surface_actor_get_window (META_SURFACE_ACTOR (source));
+        else
+          return NULL;
+      }
+    case META_EVENT_ROUTE_WAYLAND_POPUP:
+    case META_EVENT_ROUTE_WINDOW_OP:
+    case META_EVENT_ROUTE_COMPOSITOR_GRAB:
+      return display->grab_window;
     default:
-      break;
+      g_assert_not_reached ();
     }
-
-  source = clutter_event_get_source (event);
-  if (META_IS_SURFACE_ACTOR (source))
-    return meta_surface_actor_get_window (META_SURFACE_ACTOR (source));
-
-  return NULL;
 }
 
 static void
@@ -99,20 +110,82 @@ handle_idletime_for_event (const ClutterEvent *event)
 #endif /* HAVE_NATIVE_BACKEND */
 }
 
+static gboolean
+sequence_is_pointer_emulated (MetaDisplay        *display,
+                              const ClutterEvent *event)
+{
+  ClutterEventSequence *sequence;
+
+  sequence = clutter_event_get_event_sequence (event);
+
+  if (!sequence)
+    return FALSE;
+
+  if (clutter_event_is_pointer_emulated (event))
+    return TRUE;
+
+#ifdef HAVE_NATIVE_BACKEND
+  MetaBackend *backend = meta_get_backend ();
+
+  /* When using Clutter's native input backend there is no concept of
+   * pointer emulating sequence, we still must make up our own to be
+   * able to implement single-touch (hence pointer alike) behavior.
+   *
+   * This is implemented similarly to X11, where only the first touch
+   * on screen gets the "pointer emulated" flag, and it won't get assigned
+   * to another sequence until the next first touch on an idle touchscreen.
+   */
+  if (META_IS_BACKEND_NATIVE (backend))
+    {
+      MetaGestureTracker *tracker;
+
+      tracker = meta_display_get_gesture_tracker (display);
+
+      if (event->type == CLUTTER_TOUCH_BEGIN &&
+          meta_gesture_tracker_get_n_current_touches (tracker) == 0)
+        return TRUE;
+    }
+#endif /* HAVE_NATIVE_BACKEND */
+
+  return FALSE;
+}
+
+static void
+meta_display_update_pointer_emulating_sequence (MetaDisplay        *display,
+                                                const ClutterEvent *event)
+{
+  ClutterEventSequence *sequence;
+
+  sequence = clutter_event_get_event_sequence (event);
+
+  if (event->type == CLUTTER_TOUCH_BEGIN &&
+      !display->pointer_emulating_sequence &&
+      sequence_is_pointer_emulated (display, event))
+    display->pointer_emulating_sequence = sequence;
+  else if (event->type == CLUTTER_TOUCH_END &&
+           display->pointer_emulating_sequence == sequence)
+    display->pointer_emulating_sequence = NULL;
+}
+
 static gboolean
 meta_display_handle_event (MetaDisplay        *display,
                            const ClutterEvent *event)
 {
   MetaWindow *window;
-  gboolean bypass_clutter = FALSE, bypass_wayland = FALSE;
-  MetaWaylandCompositor *compositor = NULL;
+  gboolean bypass_clutter = FALSE;
+  G_GNUC_UNUSED gboolean bypass_wayland = FALSE;
   MetaGestureTracker *tracker;
 
+  meta_display_update_pointer_emulating_sequence (display, event);
+
+#ifdef HAVE_WAYLAND
+  MetaWaylandCompositor *compositor = NULL;
   if (meta_is_wayland_compositor ())
     {
       compositor = meta_wayland_compositor_get_default ();
       meta_wayland_compositor_update (compositor, event);
     }
+#endif
 
   handle_idletime_for_event (event);
 
@@ -146,13 +219,11 @@ meta_display_handle_event (MetaDisplay        *display,
 
   if (meta_gesture_tracker_handle_event (tracker, event))
     {
-      bypass_wayland = TRUE;
-      bypass_clutter = meta_gesture_tracker_consumes_event (tracker, event);
+      bypass_wayland = bypass_clutter = TRUE;
       goto out;
     }
 
-  if (display->grab_window == window &&
-      meta_grab_op_is_moving_or_resizing (display->grab_op))
+  if (display->event_route == META_EVENT_ROUTE_WINDOW_OP)
     {
       if (meta_window_handle_mouse_grab_op_event (window, event))
         {
@@ -177,29 +248,42 @@ meta_display_handle_event (MetaDisplay        *display,
 
   if (window)
     {
-      /* Swallow all events on windows that come our way. */
-      bypass_clutter = TRUE;
-
-      /* Under X11, we have a Sync grab and in order to send it back to
-       * clients, we have to explicitly replay it.
-       *
-       * Under Wayland, we retrieve all events and we have to make sure
-       * to filter them out from Wayland clients.
-       */
-      if (meta_window_handle_ungrabbed_event (window, event))
+      if (!clutter_event_get_event_sequence (event))
         {
+          /* Swallow all non-touch events on windows that come our way.
+           * Touch events that reach here aren't yet in an accepted state,
+           * so Clutter must see them to maybe trigger gestures into
+           * recognition.
+           */
+          bypass_clutter = TRUE;
+        }
+
+      meta_window_handle_ungrabbed_event (window, event);
+
+      /* This might start a grab op. If it does, then filter out the
+       * event, and if it doesn't, replay the event to release our
+       * own sync grab. */
+
+      if (display->event_route == META_EVENT_ROUTE_WINDOW_OP)
+        {
+          bypass_clutter = TRUE;
           bypass_wayland = TRUE;
         }
       else
         {
-          MetaBackend *backend = meta_get_backend ();
-          if (META_IS_BACKEND_X11 (backend))
+          /* Only replay button press events, since that's where we
+           * have the synchronous grab. */
+          if (event->type == CLUTTER_BUTTON_PRESS)
             {
-              Display *xdisplay = meta_backend_x11_get_xdisplay (META_BACKEND_X11 (backend));
-              meta_verbose ("Allowing events time %u\n",
-                            (unsigned int)event->button.time);
-              XIAllowEvents (xdisplay, clutter_event_get_device_id (event),
-                             XIReplayDevice, event->button.time);
+              MetaBackend *backend = meta_get_backend ();
+              if (META_IS_BACKEND_X11 (backend))
+                {
+                  Display *xdisplay = meta_backend_x11_get_xdisplay (META_BACKEND_X11 (backend));
+                  meta_verbose ("Allowing events time %u\n",
+                                (unsigned int)event->button.time);
+                  XIAllowEvents (xdisplay, clutter_event_get_device_id (event),
+                                 XIReplayDevice, event->button.time);
+                }
             }
         }
 
@@ -208,18 +292,20 @@ meta_display_handle_event (MetaDisplay        *display,
 
  out:
   /* If the compositor has a grab, don't pass that through to Wayland */
-  if (display->grab_op == META_GRAB_OP_COMPOSITOR)
+  if (display->event_route == META_EVENT_ROUTE_COMPOSITOR_GRAB)
     bypass_wayland = TRUE;
 
   /* If a Wayland client has a grab, don't pass that through to Clutter */
-  if (display->grab_op == META_GRAB_OP_WAYLAND_POPUP)
+  if (display->event_route == META_EVENT_ROUTE_WAYLAND_POPUP)
     bypass_clutter = TRUE;
 
+#ifdef HAVE_WAYLAND
   if (compositor && !bypass_wayland)
     {
       if (meta_wayland_compositor_handle_event (compositor, event))
         bypass_clutter = TRUE;
     }
+#endif
 
   display->current_time = CurrentTime;
   return bypass_clutter;
@@ -237,7 +323,6 @@ event_callback (const ClutterEvent *event,
 void
 meta_display_init_events (MetaDisplay *display)
 {
-  meta_display_init_events_x11 (display);
   display->clutter_event_filter = clutter_event_add_filter (NULL,
                                                             event_callback,
                                                             NULL,
@@ -247,7 +332,6 @@ meta_display_init_events (MetaDisplay *display)
 void
 meta_display_free_events (MetaDisplay *display)
 {
-  meta_display_free_events_x11 (display);
   clutter_event_remove_filter (display->clutter_event_filter);
   display->clutter_event_filter = 0;
 }
