@@ -52,6 +52,7 @@
 #include "ui.h"
 #include <meta/prefs.h>
 #include <meta/compositor.h>
+#include <meta/meta-backend.h>
 
 #include <glib-object.h>
 #include <glib-unix.h>
@@ -77,8 +78,9 @@
 
 #include "x11/session.h"
 
+#ifdef HAVE_WAYLAND
 #include "wayland/meta-wayland.h"
-#include "backends/meta-backend.h"
+#endif
 
 /*
  * The exit code we'll return to our parent process when we eventually die.
@@ -160,7 +162,9 @@ static gchar    *opt_client_id;
 static gboolean  opt_replace_wm;
 static gboolean  opt_disable_sm;
 static gboolean  opt_sync;
+#ifdef HAVE_WAYLAND
 static gboolean  opt_wayland;
+#endif
 #ifdef HAVE_NATIVE_BACKEND
 static gboolean  opt_display_server;
 #endif
@@ -201,12 +205,14 @@ static GOptionEntry meta_options[] = {
     N_("Make X calls synchronous"),
     NULL
   },
+#ifdef HAVE_WAYLAND
   {
     "wayland", 0, 0, G_OPTION_ARG_NONE,
     &opt_wayland,
     N_("Run as a wayland compositor"),
     NULL
   },
+#endif
 #ifdef HAVE_NATIVE_BACKEND
   {
     "display-server", 0, 0, G_OPTION_ARG_NONE,
@@ -271,8 +277,10 @@ meta_finalize (void)
     meta_display_close (display,
                         CurrentTime); /* I doubt correct timestamps matter here */
 
+#ifdef HAVE_WAYLAND
   if (meta_is_wayland_compositor ())
     meta_wayland_finalize ();
+#endif
 }
 
 static gboolean
@@ -319,9 +327,13 @@ meta_init (void)
 #if defined(CLUTTER_WINDOWING_EGL) && defined(HAVE_NATIVE_BACKEND)
   if (opt_display_server)
     clutter_set_windowing_backend (CLUTTER_WINDOWING_EGL);
+  else
 #endif
+    clutter_set_windowing_backend (CLUTTER_WINDOWING_X11);
 
+#ifdef HAVE_WAYLAND
   meta_set_is_wayland_compositor (opt_wayland);
+#endif
 
   if (g_get_home_dir ())
     if (chdir (g_get_home_dir ()) < 0)
@@ -334,15 +346,24 @@ meta_init (void)
   g_irepository_prepend_search_path (MUTTER_PKGLIBDIR);
 #endif
 
+#ifdef HAVE_WAYLAND
   if (meta_is_wayland_compositor ())
-    {
-      /* NB: When running as a hybrid wayland compositor we run our own headless X
-       * server so the user can't control the X display to connect too. */
-      meta_wayland_init ();
-    }
-  else
+    meta_wayland_pre_clutter_init ();
+#endif
+
+  /* NB: When running as a hybrid wayland compositor we run our own headless X
+   * server so the user can't control the X display to connect too. */
+  if (!meta_is_wayland_compositor ())
     meta_select_display (opt_display_name);
 
+  meta_clutter_init ();
+
+#ifdef HAVE_WAYLAND
+  /* Bring up Wayland. This also launches Xwayland and sets DISPLAY as well... */
+  if (meta_is_wayland_compositor ())
+    meta_wayland_init ();
+#endif
+
   meta_set_syncing (opt_sync || (g_getenv ("MUTTER_SYNC") != NULL));
 
   if (opt_replace_wm)
@@ -355,18 +376,6 @@ meta_init (void)
 
   meta_ui_init ();
 
-  /* If we are running with wayland then we don't wait until we have
-   * an X connection before initializing clutter we instead initialize
-   * it earlier since we need to initialize the GL driver so the driver
-   * can register any needed wayland extensions. */
-  if (!meta_is_wayland_compositor ())
-    {
-      /*
-       * Clutter can only be initialized after the UI.
-       */
-      meta_clutter_init ();
-    }
-
   meta_restart_init ();
 
   /*
