@@ -28,7 +28,6 @@
 
 #include <config.h>
 #include <meta/prefs.h>
-#include "ui.h"
 #include "util-private.h"
 #include "meta-plugin-manager.h"
 #include <glib.h>
@@ -67,6 +66,7 @@
 #define SCHEMA_INTERFACE       "org.gnome.desktop.interface"
 #define SCHEMA_INPUT_SOURCES   "org.gnome.desktop.input-sources"
 #define SCHEMA_XSETTINGS       "org.gnome.settings-daemon.plugins.xsettings"
+#define SCHEMA_MOUSE           "org.gnome.settings-daemon.peripherals.mouse"
 
 #define SETTINGS(s) g_hash_table_lookup (settings_schemas, (s))
 
@@ -101,6 +101,7 @@ static gboolean gnome_animations = TRUE;
 static char *cursor_theme = NULL;
 static int   cursor_size = 24;
 static int   draggable_border_width = 10;
+static int   drag_threshold;
 static gboolean resize_with_right_button = FALSE;
 static gboolean edge_tiling = FALSE;
 static gboolean force_fullscreen = TRUE;
@@ -492,6 +493,13 @@ static MetaIntPreference preferences_int[] =
       },
       &draggable_border_width
     },
+    {
+      { "drag-threshold",
+        SCHEMA_MOUSE,
+        META_PREF_DRAG_THRESHOLD,
+      },
+      &drag_threshold
+    },
     { { NULL, 0, 0 }, NULL },
   };
 
@@ -978,6 +986,10 @@ meta_prefs_init (void)
   g_signal_connect (settings, "changed", G_CALLBACK (settings_changed), NULL);
   g_hash_table_insert (settings_schemas, g_strdup (SCHEMA_MUTTER), settings);
 
+  settings = g_settings_new (SCHEMA_MOUSE);
+  g_signal_connect (settings, "changed", G_CALLBACK (settings_changed), NULL);
+  g_hash_table_insert (settings_schemas, g_strdup (SCHEMA_MOUSE), settings);
+
   /* Individual keys we watch outside of our schemas */
   settings = g_settings_new (SCHEMA_INTERFACE);
   g_signal_connect (settings, "changed::" KEY_GNOME_ACCESSIBILITY,
@@ -1198,8 +1210,9 @@ settings_changed (GSettings *settings,
     }
   else
     {
-      /* Someone added a preference of an unhandled type */
-      g_assert_not_reached ();
+      /* Unknown preference type. This quite likely simply isn't
+       * a preference we track changes to. */
+      return;
     }
 
   g_variant_unref (value);
@@ -1334,10 +1347,26 @@ meta_prefs_get_cursor_theme (void)
   return cursor_theme;
 }
 
+static int
+get_scale_factor (void)
+{
+  GdkScreen *screen;
+  GValue value = G_VALUE_INIT;
+
+  g_value_init (&value, G_TYPE_INT);
+
+  /* XXX: Should this be in ui/ ? Or MetaMonitorManager? */
+  screen = gdk_screen_get_default ();
+  if (gdk_screen_get_setting (screen, "gdk-window-scaling-factor", &value))
+    return g_value_get_int (&value);
+  else
+    return 1;
+}
+
 int
 meta_prefs_get_cursor_size (void)
 {
-  return cursor_size;
+  return cursor_size * get_scale_factor ();
 }
 
 
@@ -1668,43 +1697,43 @@ button_layout_handler (GVariant *value,
   g_strfreev (sides);
 
   /* Invert the button layout for RTL languages */
-  if (meta_ui_get_direction() == META_UI_DIRECTION_RTL)
-  {
-    MetaButtonLayout rtl_layout;
-    int j;
+  if (meta_get_locale_direction() == META_LOCALE_DIRECTION_RTL)
+    {
+      MetaButtonLayout rtl_layout;
+      int j;
 
-    for (i = 0; new_layout.left_buttons[i] != META_BUTTON_FUNCTION_LAST; i++);
-    for (j = 0; j < i; j++)
-      {
-        rtl_layout.right_buttons[j] = new_layout.left_buttons[i - j - 1];
-        if (j == 0)
-          rtl_layout.right_buttons_has_spacer[i - 1] = new_layout.left_buttons_has_spacer[i - j - 1];
-        else
-          rtl_layout.right_buttons_has_spacer[j - 1] = new_layout.left_buttons_has_spacer[i - j - 1];
-      }
-    for (; j < MAX_BUTTONS_PER_CORNER; j++)
-      {
-        rtl_layout.right_buttons[j] = META_BUTTON_FUNCTION_LAST;
-        rtl_layout.right_buttons_has_spacer[j] = FALSE;
-      }
+      for (i = 0; new_layout.left_buttons[i] != META_BUTTON_FUNCTION_LAST; i++);
+      for (j = 0; j < i; j++)
+        {
+          rtl_layout.right_buttons[j] = new_layout.left_buttons[i - j - 1];
+          if (j == 0)
+            rtl_layout.right_buttons_has_spacer[i - 1] = new_layout.left_buttons_has_spacer[i - j - 1];
+          else
+            rtl_layout.right_buttons_has_spacer[j - 1] = new_layout.left_buttons_has_spacer[i - j - 1];
+        }
+      for (; j < MAX_BUTTONS_PER_CORNER; j++)
+        {
+          rtl_layout.right_buttons[j] = META_BUTTON_FUNCTION_LAST;
+          rtl_layout.right_buttons_has_spacer[j] = FALSE;
+        }
 
-    for (i = 0; new_layout.right_buttons[i] != META_BUTTON_FUNCTION_LAST; i++);
-    for (j = 0; j < i; j++)
-      {
-        rtl_layout.left_buttons[j] = new_layout.right_buttons[i - j - 1];
-        if (j == 0)
-          rtl_layout.left_buttons_has_spacer[i - 1] = new_layout.right_buttons_has_spacer[i - j - 1];
-        else
-          rtl_layout.left_buttons_has_spacer[j - 1] = new_layout.right_buttons_has_spacer[i - j - 1];
-      }
-    for (; j < MAX_BUTTONS_PER_CORNER; j++)
-      {
-        rtl_layout.left_buttons[j] = META_BUTTON_FUNCTION_LAST;
-        rtl_layout.left_buttons_has_spacer[j] = FALSE;
-      }
+      for (i = 0; new_layout.right_buttons[i] != META_BUTTON_FUNCTION_LAST; i++);
+      for (j = 0; j < i; j++)
+        {
+          rtl_layout.left_buttons[j] = new_layout.right_buttons[i - j - 1];
+          if (j == 0)
+            rtl_layout.left_buttons_has_spacer[i - 1] = new_layout.right_buttons_has_spacer[i - j - 1];
+          else
+            rtl_layout.left_buttons_has_spacer[j - 1] = new_layout.right_buttons_has_spacer[i - j - 1];
+        }
+      for (; j < MAX_BUTTONS_PER_CORNER; j++)
+        {
+          rtl_layout.left_buttons[j] = META_BUTTON_FUNCTION_LAST;
+          rtl_layout.left_buttons_has_spacer[j] = FALSE;
+        }
 
-    new_layout = rtl_layout;
-  }
+      new_layout = rtl_layout;
+    }
 
   if (!button_layout_equal (&button_layout, &new_layout))
     {
@@ -1908,6 +1937,9 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_DRAGGABLE_BORDER_WIDTH:
       return "DRAGGABLE_BORDER_WIDTH";
 
+    case META_PREF_DRAG_THRESHOLD:
+      return "DRAG_TRHESHOLD";
+
     case META_PREF_DYNAMIC_WORKSPACES:
       return "DYNAMIC_WORKSPACES";
 
@@ -2001,23 +2033,6 @@ update_binding (MetaKeyPref *binding,
           continue;
         }
 
-      /* Bug 329676: Bindings which can be shifted must not have no modifiers,
-       * nor only SHIFT as a modifier.
-       */
-
-      if (binding->add_shift &&
-          0 != keysym &&
-          (META_VIRTUAL_SHIFT_MASK == mods || 0 == mods))
-        {
-          meta_warning ("Cannot bind \"%s\" to %s: it needs a modifier "
-                        "such as Ctrl or Alt.\n",
-                        binding->name, strokes[i]);
-
-          /* Value is kept and will thus be removed next time we save the key.
-           * Changing the key in response to a modification could lead to cyclic calls. */
-          continue;
-        }
-
       combo = g_malloc0 (sizeof (MetaKeyCombo));
       combo->keysym = keysym;
       combo->keycode = keycode;
@@ -2193,7 +2208,6 @@ meta_prefs_add_keybinding (const char           *name,
   pref->settings = g_object_ref (settings);
   pref->action = action;
   pref->combos = NULL;
-  pref->add_shift = (flags & META_KEY_BINDING_REVERSES) != 0;
   pref->per_window = (flags & META_KEY_BINDING_PER_WINDOW) != 0;
   pref->builtin = (flags & META_KEY_BINDING_BUILTIN) != 0;
 
@@ -2373,6 +2387,12 @@ meta_prefs_get_draggable_border_width (void)
   return draggable_border_width;
 }
 
+int
+meta_prefs_get_drag_threshold (void)
+{
+  return drag_threshold;
+}
+
 void
 meta_prefs_set_force_fullscreen (gboolean whether)
 {
