@@ -329,9 +329,6 @@ struct _MetaWindow
    */
   guint calc_placement : 1;
 
-  /* Transient parent is a root window */
-  guint transient_parent_is_root_window : 1;
-
   /* if TRUE, window was maximized at start of current grab op */
   guint shaken_loose : 1;
 
@@ -628,7 +625,7 @@ void meta_window_set_user_time (MetaWindow *window,
                                 guint32     timestamp);
 
 void meta_window_update_for_monitors_changed (MetaWindow *window);
-void meta_window_update_on_all_workspaces (MetaWindow *window);
+void meta_window_on_all_workspaces_changed (MetaWindow *window);
 
 gboolean meta_window_should_attach_to_parent (MetaWindow *window);
 gboolean meta_window_can_tile_side_by_side   (MetaWindow *window);
@@ -665,8 +662,8 @@ void meta_window_handle_enter (MetaWindow  *window,
                                guint        root_y);
 void meta_window_handle_leave (MetaWindow  *window);
 
-gboolean meta_window_handle_ungrabbed_event (MetaWindow         *window,
-                                             const ClutterEvent *event);
+void meta_window_handle_ungrabbed_event (MetaWindow         *window,
+                                         const ClutterEvent *event);
 
 void meta_window_get_client_area_rect (const MetaWindow      *window,
                                        cairo_rectangle_int_t *rect);
