@@ -47,12 +47,6 @@
 #include <canberra-gtk.h>
 #endif
 
-enum {
-  PROP_0,
-
-  PROP_N_WINDOWS,
-};
-
 void meta_workspace_queue_calc_showing   (MetaWorkspace *workspace);
 static void focus_ancestor_or_top_window (MetaWorkspace *workspace,
                                           MetaWindow    *not_this_one,
@@ -62,6 +56,17 @@ static void free_this                    (gpointer candidate,
 
 G_DEFINE_TYPE (MetaWorkspace, meta_workspace, G_TYPE_OBJECT);
 
+enum {
+  PROP_0,
+
+  PROP_N_WINDOWS,
+  PROP_WORKSPACE_INDEX,
+
+  LAST_PROP,
+};
+
+static GParamSpec *obj_props[LAST_PROP];
+
 enum
 {
   WINDOW_ADDED,
@@ -70,7 +75,7 @@ enum
   LAST_SIGNAL
 };
 
-static guint signals [LAST_SIGNAL] = { 0 };
+static guint signals[LAST_SIGNAL] = { 0 };
 
 static void
 meta_workspace_finalize (GObject *object)
@@ -110,6 +115,9 @@ meta_workspace_get_property (GObject      *object,
        */
       g_value_set_uint (value, g_list_length (ws->windows));
       break;
+    case PROP_WORKSPACE_INDEX:
+      g_value_set_uint (value, meta_workspace_index (ws));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -120,8 +128,6 @@ static void
 meta_workspace_class_init (MetaWorkspaceClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GParamSpec   *pspec;
-
   object_class->finalize     = meta_workspace_finalize;
   object_class->get_property = meta_workspace_get_property;
   object_class->set_property = meta_workspace_set_property;
@@ -141,15 +147,18 @@ meta_workspace_class_init (MetaWorkspaceClass *klass)
                                           G_TYPE_NONE, 1,
                                           META_TYPE_WINDOW);
 
-  pspec = g_param_spec_uint ("n-windows",
-                             "N Windows",
-                             "Number of windows",
-                             0, G_MAXUINT, 0,
-                             G_PARAM_READABLE);
+  obj_props[PROP_N_WINDOWS] = g_param_spec_uint ("n-windows",
+                                                 "N Windows",
+                                                 "Number of windows",
+                                                 0, G_MAXUINT, 0,
+                                                 G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  obj_props[PROP_WORKSPACE_INDEX] = g_param_spec_uint ("workspace-index",
+                                                       "Workspace index",
+                                                       "The workspace's index",
+                                                       0, G_MAXUINT, 0,
+                                                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
-  g_object_class_install_property (object_class,
-                                   PROP_N_WINDOWS,
-                                   pspec);
+  g_object_class_install_properties (object_class, LAST_PROP, obj_props);
 }
 
 static void
@@ -157,15 +166,6 @@ meta_workspace_init (MetaWorkspace *workspace)
 {
 }
 
-static void
-maybe_add_to_list (MetaScreen *screen, MetaWindow *window, gpointer data)
-{
-  GList **mru_list = data;
-
-  if (window->on_all_workspaces)
-    *mru_list = g_list_prepend (*mru_list, window);
-}
-
 MetaWorkspace*
 meta_workspace_new (MetaScreen *screen)
 {
@@ -178,7 +178,6 @@ meta_workspace_new (MetaScreen *screen)
     g_list_append (workspace->screen->workspaces, workspace);
   workspace->windows = NULL;
   workspace->mru_list = NULL;
-  meta_screen_foreach_window (screen, maybe_add_to_list, &workspace->mru_list);
 
   workspace->work_areas_invalid = TRUE;
   workspace->work_area_monitor = NULL;
@@ -245,30 +244,11 @@ workspace_free_builtin_struts (MetaWorkspace *workspace)
 void
 meta_workspace_remove (MetaWorkspace *workspace)
 {
-  GList *tmp;
   MetaScreen *screen;
   int i;
 
   g_return_if_fail (workspace != workspace->screen->active_workspace);
 
-  /* Here we assume all the windows are already on another workspace
-   * as well, so they won't be "orphaned"
-   */
-
-  tmp = workspace->windows;
-  while (tmp != NULL)
-    {
-      GList *next;
-      MetaWindow *window = tmp->data;
-      next = tmp->next;
-
-      /* pop front of list we're iterating over */
-      meta_workspace_remove_window (workspace, window);
-      g_assert (window->workspace != NULL);
-
-      tmp = next;
-    }
-
   g_assert (workspace->windows == NULL);
 
   screen = workspace->screen;
@@ -312,38 +292,11 @@ void
 meta_workspace_add_window (MetaWorkspace *workspace,
                            MetaWindow    *window)
 {
-  g_return_if_fail (window->workspace == NULL);
-
-  /* If the window is on all workspaces, we want to add it to all mru
-   * lists, otherwise just add it to this workspaces mru list
-   */
-  if (window->on_all_workspaces)
-    {
-      if (window->workspace == NULL)
-        {
-          GList* tmp = window->screen->workspaces;
-          while (tmp)
-            {
-              MetaWorkspace* work = (MetaWorkspace*) tmp->data;
-              if (!g_list_find (work->mru_list, window))
-                work->mru_list = g_list_prepend (work->mru_list, window);
-
-              tmp = tmp->next;
-            }
-        }
-    }
-  else
-    {
-      g_assert (g_list_find (workspace->mru_list, window) == NULL);
-      workspace->mru_list = g_list_prepend (workspace->mru_list, window);
-    }
+  g_assert (g_list_find (workspace->mru_list, window) == NULL);
+  workspace->mru_list = g_list_prepend (workspace->mru_list, window);
 
   workspace->windows = g_list_prepend (workspace->windows, window);
 
-  window->workspace = workspace;
-
-  meta_window_current_workspace_changed (window);
-
   if (window->struts)
     {
       meta_topic (META_DEBUG_WORKAREA,
@@ -352,46 +305,18 @@ meta_workspace_add_window (MetaWorkspace *workspace,
       meta_workspace_invalidate_work_area (workspace);
     }
 
-  /* queue a move_resize since changing workspaces may change
-   * the relevant struts
-   */
-  meta_window_queue (window, META_QUEUE_CALC_SHOWING|META_QUEUE_MOVE_RESIZE);
-
   g_signal_emit (workspace, signals[WINDOW_ADDED], 0, window);
-  g_object_notify (G_OBJECT (workspace), "n-windows");
+  g_object_notify_by_pspec (G_OBJECT (workspace), obj_props[PROP_N_WINDOWS]);
 }
 
 void
 meta_workspace_remove_window (MetaWorkspace *workspace,
                               MetaWindow    *window)
 {
-  g_return_if_fail (window->workspace == workspace);
-
   workspace->windows = g_list_remove (workspace->windows, window);
-  window->workspace = NULL;
 
-  /* If the window is on all workspaces, we don't want to remove it
-   * from the MRU list unless this causes it to be removed from all
-   * workspaces
-   */
-  if (window->on_all_workspaces)
-    {
-      GList* tmp = window->screen->workspaces;
-      while (tmp)
-        {
-          MetaWorkspace* work = (MetaWorkspace*) tmp->data;
-          work->mru_list = g_list_remove (work->mru_list, window);
-
-          tmp = tmp->next;
-        }
-    }
-  else
-    {
-      workspace->mru_list = g_list_remove (workspace->mru_list, window);
-      g_assert (g_list_find (workspace->mru_list, window) == NULL);
-    }
-
-  meta_window_current_workspace_changed (window);
+  workspace->mru_list = g_list_remove (workspace->mru_list, window);
+  g_assert (g_list_find (workspace->mru_list, window) == NULL);
 
   if (window->struts)
     {
@@ -401,11 +326,6 @@ meta_workspace_remove_window (MetaWorkspace *workspace,
       meta_workspace_invalidate_work_area (workspace);
     }
 
-  /* queue a move_resize since changing workspaces may change
-   * the relevant struts
-   */
-  meta_window_queue (window, META_QUEUE_CALC_SHOWING|META_QUEUE_MOVE_RESIZE);
-
   g_signal_emit (workspace, signals[WINDOW_REMOVED], 0, window);
   g_object_notify (G_OBJECT (workspace), "n-windows");
 }
@@ -414,23 +334,17 @@ void
 meta_workspace_relocate_windows (MetaWorkspace *workspace,
                                  MetaWorkspace *new_home)
 {
-  GList *tmp;
-  GList *copy;
+  GList *copy, *l;
 
   g_return_if_fail (workspace != new_home);
 
   /* can't modify list we're iterating over */
   copy = g_list_copy (workspace->windows);
 
-  tmp = copy;
-  while (tmp != NULL)
+  for (l = copy; l != NULL; l = l->next)
     {
-      MetaWindow *window = tmp->data;
-
-      meta_workspace_remove_window (workspace, window);
-      meta_workspace_add_window (new_home, window);
-
-      tmp = tmp->next;
+      MetaWindow *window = l->data;
+      meta_window_change_workspace (window, new_home);
     }
 
   g_list_free (copy);
@@ -441,15 +355,10 @@ meta_workspace_relocate_windows (MetaWorkspace *workspace,
 void
 meta_workspace_queue_calc_showing  (MetaWorkspace *workspace)
 {
-  GList *tmp;
+  GList *l;
 
-  tmp = workspace->windows;
-  while (tmp != NULL)
-    {
-      meta_window_queue (tmp->data, META_QUEUE_CALC_SHOWING);
-
-      tmp = tmp->next;
-    }
+  for (l = workspace->windows; l != NULL; l = l->next)
+    meta_window_queue (l->data, META_QUEUE_CALC_SHOWING);
 }
 
 static void
@@ -572,22 +481,18 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
    * or the new one *but not both*, then update the
    * _net_showing_desktop hint
    */
-  if (old && (old->showing_desktop ^ workspace->showing_desktop))
+  if (old && (old->showing_desktop != workspace->showing_desktop))
     meta_screen_update_showing_desktop_hint (workspace->screen);
 
   if (old == NULL)
     return;
 
   move_window = NULL;
-  if (workspace->screen->display->grab_op == META_GRAB_OP_MOVING ||
-      workspace->screen->display->grab_op == META_GRAB_OP_KEYBOARD_MOVING)
+  if (meta_grab_op_is_moving (workspace->screen->display->grab_op))
     move_window = workspace->screen->display->grab_window;
 
   if (move_window != NULL)
     {
-      if (move_window->on_all_workspaces)
-        move_window = NULL; /* don't move it after all */
-
       /* We put the window on the new workspace, flip spaces,
        * then remove from old workspace, so the window
        * never gets unmapped and we maintain the button grab
@@ -595,21 +500,13 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
        *
        * \bug  This comment appears to be the reverse of what happens
        */
-      if (move_window && (move_window->workspace != workspace))
-        {
-          meta_workspace_remove_window (old, move_window);
-          meta_workspace_add_window (workspace, move_window);
-        }
+      if (!meta_window_located_on_workspace (move_window, workspace))
+        meta_window_change_workspace (move_window, workspace);
     }
 
   meta_workspace_queue_calc_showing (old);
   meta_workspace_queue_calc_showing (workspace);
 
-  /* FIXME: Why do we need this?!?  Isn't it handled in the lines above? */
-  if (move_window)
-      /* Removes window from other spaces */
-      meta_window_change_workspace (move_window, workspace);
-
    /*
     * Notify the compositor that the active workspace is changing.
     */
@@ -627,7 +524,7 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
    meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
                                       new_space, &layout2);
 
-   if (meta_ui_get_direction() == META_UI_DIRECTION_RTL)
+   if (meta_get_locale_direction () == META_LOCALE_DIRECTION_RTL)
      {
        if (layout1.current_col > layout2.current_col)
          direction = META_MOTION_RIGHT;
@@ -711,17 +608,16 @@ meta_workspace_index (MetaWorkspace *workspace)
 }
 
 void
-meta_workspace_update_window_hints (MetaWorkspace *workspace)
+meta_workspace_index_changed (MetaWorkspace *workspace)
 {
-  GList *l = workspace->windows;
-  while (l)
+  GList *l;
+  for (l = workspace->windows; l != NULL; l = l->next)
     {
       MetaWindow *win = l->data;
-
       meta_window_current_workspace_changed (win);
-
-      l = l->next;
     }
+
+  g_object_notify_by_pspec (G_OBJECT (workspace), obj_props[PROP_WORKSPACE_INDEX]);
 }
 
 /**
@@ -736,24 +632,20 @@ meta_workspace_update_window_hints (MetaWorkspace *workspace)
 GList*
 meta_workspace_list_windows (MetaWorkspace *workspace)
 {
-  GSList *display_windows;
-  GSList *tmp;
+  GSList *display_windows, *l;
   GList *workspace_windows;
 
   display_windows = meta_display_list_windows (workspace->screen->display,
                                                META_LIST_DEFAULT);
 
   workspace_windows = NULL;
-  tmp = display_windows;
-  while (tmp != NULL)
+  for (l = display_windows; l != NULL; l = l->next)
     {
-      MetaWindow *window = tmp->data;
+      MetaWindow *window = l->data;
 
       if (meta_window_located_on_workspace (window, workspace))
         workspace_windows = g_list_prepend (workspace_windows,
                                             window);
-
-      tmp = tmp->next;
     }
 
   g_slist_free (display_windows);
@@ -764,8 +656,7 @@ meta_workspace_list_windows (MetaWorkspace *workspace)
 void
 meta_workspace_invalidate_work_area (MetaWorkspace *workspace)
 {
-  GList *tmp;
-  GList *windows;
+  GList *windows, *l;
   int i;
 
   if (workspace->work_areas_invalid)
@@ -805,14 +696,11 @@ meta_workspace_invalidate_work_area (MetaWorkspace *workspace)
 
   /* redo the size/position constraints on all windows */
   windows = meta_workspace_list_windows (workspace);
-  tmp = windows;
-  while (tmp != NULL)
+
+  for (l = windows; l != NULL; l = l->next)
     {
-      MetaWindow *w = tmp->data;
-
+      MetaWindow *w = l->data;
       meta_window_queue (w, META_QUEUE_MOVE_RESIZE);
-
-      tmp = tmp->next;
     }
 
   g_list_free (windows);
@@ -831,11 +719,8 @@ copy_strut_list(GSList *original)
 {
   GSList *result = NULL;
 
-  while (original)
-    {
-      result = g_slist_prepend (result, copy_strut (original->data));
-      original = original->next;
-    }
+  for (; original != NULL; original = original->next)
+    result = g_slist_prepend (result, copy_strut (original->data));
 
   return g_slist_reverse (result);
 }
@@ -1202,7 +1087,7 @@ meta_workspace_get_neighbor (MetaWorkspace      *workspace,
   meta_verbose ("Getting neighbor of %d in direction %s\n",
                 current_space, meta_motion_direction_to_string (direction));
 
-  ltr = meta_ui_get_direction() == META_UI_DIRECTION_LTR;
+  ltr = (meta_get_locale_direction () == META_LOCALE_DIRECTION_LTR);
 
   switch (direction)
     {
