@@ -331,6 +331,17 @@ meta_ui_get_frame_borders (MetaUI *ui,
                            borders);
 }
 
+static void
+set_background_none (Display *xdisplay,
+                     Window   xwindow)
+{
+  XSetWindowAttributes attrs;
+
+  attrs.background_pixmap = None;
+  XChangeWindowAttributes (xdisplay, xwindow,
+                           CWBackPixmap, &attrs);
+}
+
 Window
 meta_ui_create_frame_window (MetaUI *ui,
                              Display *xdisplay,
@@ -397,6 +408,7 @@ meta_ui_create_frame_window (MetaUI *ui,
 		    &attrs, attributes_mask);
 
   gdk_window_resize (window, width, height);
+  set_background_none (xdisplay, GDK_WINDOW_XID (window));
 
   meta_frames_manage_window (ui->frames, GDK_WINDOW_XID (window), window);
 
@@ -449,16 +461,6 @@ meta_ui_unmap_frame (MetaUI *ui,
     gdk_window_hide (window);
 }
 
-void
-meta_ui_unflicker_frame_bg (MetaUI *ui,
-                            Window  xwindow,
-                            int     target_width,
-                            int     target_height)
-{
-  meta_frames_unflicker_bg (ui->frames, xwindow,
-                            target_width, target_height);
-}
-
 void
 meta_ui_update_frame_style (MetaUI  *ui,
                             Window   xwindow)
@@ -473,13 +475,6 @@ meta_ui_repaint_frame (MetaUI *ui,
   meta_frames_repaint_frame (ui->frames, xwindow);
 }
 
-void
-meta_ui_reset_frame_bg (MetaUI *ui,
-                        Window xwindow)
-{
-  meta_frames_reset_bg (ui->frames, xwindow);
-}
-
 cairo_region_t *
 meta_ui_get_frame_bounds (MetaUI  *ui,
                           Window   xwindow,
@@ -658,27 +653,3 @@ meta_ui_window_is_widget (MetaUI *ui,
   else
     return FALSE;
 }
-
-int
-meta_ui_get_drag_threshold (MetaUI *ui)
-{
-  GtkSettings *settings;
-  int threshold;
-
-  settings = gtk_widget_get_settings (GTK_WIDGET (ui->frames));
-
-  threshold = 8;
-  g_object_get (G_OBJECT (settings), "gtk-dnd-drag-threshold", &threshold, NULL);
-
-  return threshold;
-}
-
-MetaUIDirection
-meta_ui_get_direction (void)
-{
-  if (gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL)
-    return META_UI_DIRECTION_RTL;
-
-  return META_UI_DIRECTION_LTR;
-}
-
