@@ -246,14 +246,25 @@ sync_focus_surface (MetaWaylandPointer *pointer)
   MetaDisplay *display = meta_get_display ();
   MetaWaylandSurface *focus_surface;
 
-  /* Don't update the focus surface while we have a move/resize grab. */
-  if (meta_grab_op_is_moving_or_resizing (display->grab_op))
-    return;
+  switch (display->event_route)
+    {
+    case META_EVENT_ROUTE_WINDOW_OP:
+      /* Don't update the focus surface while we're grabbing a window. */
+      return;
 
-  if (meta_grab_op_should_block_wayland (display->grab_op))
-    focus_surface = NULL;
-  else
-    focus_surface = pointer->current;
+    case META_EVENT_ROUTE_COMPOSITOR_GRAB:
+      /* The compositor has focus, so remove our focus... */
+      focus_surface = NULL;
+      break;
+
+    case META_EVENT_ROUTE_NORMAL:
+    case META_EVENT_ROUTE_WAYLAND_POPUP:
+      focus_surface = pointer->current;
+      break;
+
+    default:
+      g_assert_not_reached ();
+    }
 
   if (focus_surface != pointer->focus_surface)
     {
@@ -802,8 +813,7 @@ meta_wayland_pointer_create_new_resource (MetaWaylandPointer *pointer,
 {
   struct wl_resource *cr;
 
-  cr = wl_resource_create (client, &wl_pointer_interface,
-			   MIN (META_WL_POINTER_VERSION, wl_resource_get_version (seat_resource)), id);
+  cr = wl_resource_create (client, &wl_pointer_interface, wl_resource_get_version (seat_resource), id);
   wl_resource_set_implementation (cr, &pointer_interface, pointer, unbind_resource);
   wl_list_insert (&pointer->resource_list, wl_resource_get_link (cr));
 
