@@ -212,7 +212,12 @@ toplevel_surface_commit (MetaWaylandSurface      *surface,
   /* We resize X based surfaces according to X events */
   if (window->client_type == META_WINDOW_CLIENT_TYPE_WAYLAND)
     {
-      MetaRectangle geom;
+      MetaRectangle geom = { 0 };
+
+      CoglTexture *texture = surface->buffer->texture;
+      /* Update the buffer rect immediately. */
+      window->buffer_rect.width = cogl_texture_get_width (texture);
+      window->buffer_rect.height = cogl_texture_get_height (texture);
 
       if (pending->has_new_geometry)
         {
@@ -243,7 +248,10 @@ toplevel_surface_commit (MetaWaylandSurface      *surface,
           return;
         }
 
-      meta_window_wayland_move_resize (window, geom, pending->dx, pending->dy);
+      meta_window_wayland_move_resize (window,
+                                       &surface->acked_configure_serial,
+                                       geom, pending->dx, pending->dy);
+      surface->acked_configure_serial.set = FALSE;
     }
 }
 
@@ -598,7 +606,7 @@ wl_surface_set_buffer_scale (struct wl_client *client,
     g_warning ("Trying to set invalid buffer_scale of %d\n", scale);
 }
 
-const struct wl_surface_interface meta_wayland_wl_surface_interface = {
+static const struct wl_surface_interface meta_wayland_wl_surface_interface = {
   wl_surface_destroy,
   wl_surface_attach,
   wl_surface_damage,
@@ -688,8 +696,7 @@ meta_wayland_surface_create (MetaWaylandCompositor *compositor,
   surface->compositor = compositor;
   surface->scale = 1;
 
-  surface->resource = wl_resource_create (client, &wl_surface_interface,
-                                          MIN (META_WL_SURFACE_VERSION, wl_resource_get_version (compositor_resource)), id);
+  surface->resource = wl_resource_create (client, &wl_surface_interface, wl_resource_get_version (compositor_resource), id);
   wl_resource_set_implementation (surface->resource, &meta_wayland_wl_surface_interface, surface, wl_surface_destructor);
 
   surface->buffer_destroy_listener.notify = surface_handle_buffer_destroy;
@@ -707,7 +714,6 @@ destroy_surface_extension (MetaWaylandSurfaceExtension *extension)
 
 static gboolean
 create_surface_extension (MetaWaylandSurfaceExtension *extension,
-                          int                          max_version,
                           const struct wl_interface   *interface,
                           const void                  *implementation,
                           wl_resource_destroy_func_t   destructor,
@@ -721,8 +727,7 @@ create_surface_extension (MetaWaylandSurfaceExtension *extension,
     return FALSE;
 
   client = wl_resource_get_client (surface->resource);
-  extension->resource = wl_resource_create (client, interface,
-                                            MIN (max_version, wl_resource_get_version (master_resource)), id);
+  extension->resource = wl_resource_create (client, interface, wl_resource_get_version (master_resource), id);
   wl_resource_set_implementation (extension->resource, implementation, surface, destructor);
 
   return TRUE;
@@ -811,16 +816,20 @@ xdg_surface_show_window_menu (struct wl_client *client,
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
   MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
 
-  if (!meta_wayland_seat_can_grab_surface (seat, surface, serial))
+  if (!meta_wayland_seat_get_grab_info (seat, surface, serial, NULL, NULL))
     return;
 
-  meta_window_show_menu (surface->window, META_WINDOW_MENU_WM, x, y);
+  meta_window_show_menu (surface->window, META_WINDOW_MENU_WM,
+                         surface->window->buffer_rect.x + x,
+                         surface->window->buffer_rect.y + y);
 }
 
 static gboolean
 begin_grab_op_on_surface (MetaWaylandSurface *surface,
                           MetaWaylandSeat    *seat,
-                          MetaGrabOp          grab_op)
+                          MetaGrabOp          grab_op,
+                          gfloat              x,
+                          gfloat              y)
 {
   MetaWindow *window = surface->window;
 
@@ -836,8 +845,7 @@ begin_grab_op_on_surface (MetaWaylandSurface *surface,
                                      1, /* button. XXX? */
                                      0, /* modmask */
                                      meta_display_get_current_time_roundtrip (window->display),
-                                     seat->pointer.grab_x,
-                                     seat->pointer.grab_y);
+                                     x, y);
 }
 
 static void
@@ -848,38 +856,35 @@ xdg_surface_move (struct wl_client *client,
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
   MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+  gfloat x, y;
 
-  if (!meta_wayland_seat_can_grab_surface (seat, surface, serial))
+  if (!meta_wayland_seat_get_grab_info (seat, surface, serial, &x, &y))
     return;
 
-  begin_grab_op_on_surface (surface, seat, META_GRAB_OP_MOVING);
+  begin_grab_op_on_surface (surface, seat, META_GRAB_OP_MOVING, x, y);
 }
 
 static MetaGrabOp
 grab_op_for_xdg_surface_resize_edge (int edge)
 {
-  switch (edge)
+  MetaGrabOp op = META_GRAB_OP_WINDOW_BASE;
+
+  if (edge & XDG_SURFACE_RESIZE_EDGE_TOP)
+    op |= META_GRAB_OP_WINDOW_DIR_NORTH;
+  if (edge & XDG_SURFACE_RESIZE_EDGE_BOTTOM)
+    op |= META_GRAB_OP_WINDOW_DIR_SOUTH;
+  if (edge & XDG_SURFACE_RESIZE_EDGE_LEFT)
+    op |= META_GRAB_OP_WINDOW_DIR_WEST;
+  if (edge & XDG_SURFACE_RESIZE_EDGE_RIGHT)
+    op |= META_GRAB_OP_WINDOW_DIR_EAST;
+
+  if (op == META_GRAB_OP_WINDOW_BASE)
     {
-    case XDG_SURFACE_RESIZE_EDGE_TOP_LEFT:
-      return META_GRAB_OP_RESIZING_NW;
-    case XDG_SURFACE_RESIZE_EDGE_TOP:
-      return META_GRAB_OP_RESIZING_N;
-    case XDG_SURFACE_RESIZE_EDGE_TOP_RIGHT:
-      return META_GRAB_OP_RESIZING_NE;
-    case XDG_SURFACE_RESIZE_EDGE_RIGHT:
-      return META_GRAB_OP_RESIZING_E;
-    case XDG_SURFACE_RESIZE_EDGE_BOTTOM_RIGHT:
-      return META_GRAB_OP_RESIZING_SE;
-    case XDG_SURFACE_RESIZE_EDGE_BOTTOM:
-      return META_GRAB_OP_RESIZING_S;
-    case XDG_SURFACE_RESIZE_EDGE_BOTTOM_LEFT:
-      return META_GRAB_OP_RESIZING_SW;
-    case XDG_SURFACE_RESIZE_EDGE_LEFT:
-      return META_GRAB_OP_RESIZING_W;
-    default:
       g_warning ("invalid edge: %d", edge);
       return META_GRAB_OP_NONE;
     }
+
+  return op;
 }
 
 static void
@@ -891,11 +896,12 @@ xdg_surface_resize (struct wl_client *client,
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
   MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+  gfloat x, y;
 
-  if (!meta_wayland_seat_can_grab_surface (seat, surface, serial))
+  if (!meta_wayland_seat_get_grab_info (seat, surface, serial, &x, &y))
     return;
 
-  begin_grab_op_on_surface (surface, seat, grab_op_for_xdg_surface_resize_edge (edges));
+  begin_grab_op_on_surface (surface, seat, grab_op_for_xdg_surface_resize_edge (edges), x, y);
 }
 
 static void
@@ -903,11 +909,10 @@ xdg_surface_ack_configure (struct wl_client *client,
                            struct wl_resource *resource,
                            uint32_t serial)
 {
-  /* Do nothing for now. In the future, we'd imagine that
-   * we'd ignore attaches when we have a state pending that
-   * we haven't had the client ACK'd, to prevent a race
-   * condition when we have an in-flight attach when the
-   * client gets the new state. */
+  MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+
+  surface->acked_configure_serial.set = TRUE;
+  surface->acked_configure_serial.value = serial;
 }
 
 static void
@@ -992,7 +997,6 @@ xdg_shell_get_xdg_surface (struct wl_client *client,
   MetaWindow *window;
 
   if (!create_surface_extension (&surface->xdg_surface,
-                                 META_XDG_SURFACE_VERSION,
                                  &xdg_surface_interface,
                                  &meta_wayland_xdg_surface_interface,
                                  xdg_surface_destructor,
@@ -1049,7 +1053,6 @@ xdg_shell_get_xdg_popup (struct wl_client *client,
     return;
 
   if (!create_surface_extension (&surface->xdg_popup,
-                                 META_XDG_POPUP_VERSION,
                                  &xdg_popup_interface,
                                  &meta_wayland_xdg_popup_interface,
                                  xdg_popup_destructor,
@@ -1126,7 +1129,7 @@ bind_xdg_shell (struct wl_client *client,
 
   xdg_shell = g_slice_new (XdgShell);
 
-  xdg_shell->resource = wl_resource_create (client, &xdg_shell_interface, META_XDG_SHELL_VERSION, id);
+  xdg_shell->resource = wl_resource_create (client, &xdg_shell_interface, version, id);
   wl_resource_set_implementation (xdg_shell->resource, &meta_wayland_xdg_shell_interface, data, NULL);
 
   xdg_shell->client_destroy_listener.notify = xdg_shell_handle_client_destroy;
@@ -1159,38 +1162,35 @@ wl_shell_surface_move (struct wl_client *client,
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
   MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+  gfloat x, y;
 
-  if (!meta_wayland_seat_can_grab_surface (seat, surface, serial))
+  if (!meta_wayland_seat_get_grab_info (seat, surface, serial, &x, &y))
     return;
 
-  begin_grab_op_on_surface (surface, seat, META_GRAB_OP_MOVING);
+  begin_grab_op_on_surface (surface, seat, META_GRAB_OP_MOVING, x, y);
 }
 
 static MetaGrabOp
 grab_op_for_wl_shell_surface_resize_edge (int edge)
 {
-  switch (edge)
+  MetaGrabOp op = META_GRAB_OP_WINDOW_BASE;
+
+  if (edge & WL_SHELL_SURFACE_RESIZE_TOP)
+    op |= META_GRAB_OP_WINDOW_DIR_NORTH;
+  if (edge & WL_SHELL_SURFACE_RESIZE_BOTTOM)
+    op |= META_GRAB_OP_WINDOW_DIR_SOUTH;
+  if (edge & WL_SHELL_SURFACE_RESIZE_LEFT)
+    op |= META_GRAB_OP_WINDOW_DIR_WEST;
+  if (edge & WL_SHELL_SURFACE_RESIZE_RIGHT)
+    op |= META_GRAB_OP_WINDOW_DIR_EAST;
+
+  if (op == META_GRAB_OP_WINDOW_BASE)
     {
-    case WL_SHELL_SURFACE_RESIZE_TOP_LEFT:
-      return META_GRAB_OP_RESIZING_NW;
-    case WL_SHELL_SURFACE_RESIZE_TOP:
-      return META_GRAB_OP_RESIZING_N;
-    case WL_SHELL_SURFACE_RESIZE_TOP_RIGHT:
-      return META_GRAB_OP_RESIZING_NE;
-    case WL_SHELL_SURFACE_RESIZE_RIGHT:
-      return META_GRAB_OP_RESIZING_E;
-    case WL_SHELL_SURFACE_RESIZE_BOTTOM_RIGHT:
-      return META_GRAB_OP_RESIZING_SE;
-    case WL_SHELL_SURFACE_RESIZE_BOTTOM:
-      return META_GRAB_OP_RESIZING_S;
-    case WL_SHELL_SURFACE_RESIZE_BOTTOM_LEFT:
-      return META_GRAB_OP_RESIZING_SW;
-    case WL_SHELL_SURFACE_RESIZE_LEFT:
-      return META_GRAB_OP_RESIZING_W;
-    default:
       g_warning ("invalid edge: %d", edge);
       return META_GRAB_OP_NONE;
     }
+
+  return op;
 }
 
 static void
@@ -1202,11 +1202,12 @@ wl_shell_surface_resize (struct wl_client *client,
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (seat_resource);
   MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
+  gfloat x, y;
 
-  if (!meta_wayland_seat_can_grab_surface (seat, surface, serial))
+  if (!meta_wayland_seat_get_grab_info (seat, surface, serial, &x, &y))
     return;
 
-  begin_grab_op_on_surface (surface, seat, grab_op_for_wl_shell_surface_resize_edge (edges));
+  begin_grab_op_on_surface (surface, seat, grab_op_for_wl_shell_surface_resize_edge (edges), x, y);
 }
 
 typedef enum {
@@ -1349,7 +1350,6 @@ wl_shell_get_shell_surface (struct wl_client *client,
   MetaWindow *window;
 
   if (!create_surface_extension (&surface->wl_shell_surface,
-                                 META_WL_SHELL_SURFACE_VERSION,
                                  &wl_shell_surface_interface,
                                  &meta_wayland_wl_shell_surface_interface,
                                  wl_shell_surface_destructor,
@@ -1377,8 +1377,7 @@ bind_wl_shell (struct wl_client *client,
 {
   struct wl_resource *resource;
 
-  resource = wl_resource_create (client, &wl_shell_interface,
-                                 MIN (META_WL_SHELL_VERSION, version), id);
+  resource = wl_resource_create (client, &wl_shell_interface, version, id);
   wl_resource_set_implementation (resource, &meta_wayland_wl_shell_interface, data, NULL);
 }
 
@@ -1431,7 +1430,6 @@ get_gtk_surface (struct wl_client *client,
   MetaWaylandSurface *surface = wl_resource_get_user_data (surface_resource);
 
   if (!create_surface_extension (&surface->gtk_surface,
-                                 META_GTK_SURFACE_VERSION,
                                  &gtk_surface_interface,
                                  &meta_wayland_gtk_surface_interface,
                                  gtk_surface_destructor,
@@ -1456,8 +1454,7 @@ bind_gtk_shell (struct wl_client *client,
 {
   struct wl_resource *resource;
 
-  resource = wl_resource_create (client, &gtk_shell_interface,
-                                 MIN (META_GTK_SHELL_VERSION, version), id);
+  resource = wl_resource_create (client, &gtk_shell_interface, version, id);
   wl_resource_set_implementation (resource, &meta_wayland_gtk_shell_interface, data, NULL);
 
   /* FIXME: ask the plugin */
@@ -1705,7 +1702,6 @@ wl_subcompositor_get_subsurface (struct wl_client *client,
   MetaWaylandSurface *parent = wl_resource_get_user_data (parent_resource);
 
   if (!create_surface_extension (&surface->subsurface,
-                                 META_GTK_SURFACE_VERSION,
                                  &wl_subsurface_interface,
                                  &meta_wayland_subsurface_interface,
                                  wl_subsurface_destructor,
@@ -1743,8 +1739,7 @@ bind_subcompositor (struct wl_client *client,
 {
   struct wl_resource *resource;
 
-  resource = wl_resource_create (client, &wl_subcompositor_interface,
-                                 MIN (META_WL_SUBCOMPOSITOR_VERSION, version), id);
+  resource = wl_resource_create (client, &wl_subcompositor_interface, version, id);
   wl_resource_set_implementation (resource, &meta_wayland_subcompositor_interface, data, NULL);
 }
 
@@ -1806,7 +1801,8 @@ fill_states (struct wl_array *states, MetaWindow *window)
 void
 meta_wayland_surface_configure_notify (MetaWaylandSurface *surface,
                                        int                 new_width,
-                                       int                 new_height)
+                                       int                 new_height,
+                                       MetaWaylandSerial  *sent_serial)
 {
   if (surface->xdg_surface.resource)
     {
@@ -1827,10 +1823,23 @@ meta_wayland_surface_configure_notify (MetaWaylandSurface *surface,
       xdg_surface_send_configure (surface->xdg_surface.resource, new_width, new_height, &states, serial);
 
       wl_array_release (&states);
+
+      if (sent_serial)
+        {
+          sent_serial->set = TRUE;
+          sent_serial->value = serial;
+        }
+    }
+  else if (surface->xdg_popup.resource)
+    {
+      /* This can happen if the popup window loses or receives focus.
+       * Just ignore it. */
     }
   else if (surface->wl_shell_surface.resource)
     wl_shell_surface_send_configure (surface->wl_shell_surface.resource,
                                      0, new_width, new_height);
+  else
+    g_assert_not_reached ();
 }
 
 void
