@@ -46,6 +46,8 @@ struct _MetaWaylandTouchInfo
   MetaWaylandTouchSurface *touch_surface;
   guint32 slot_serial;
   gint32 slot;
+  gfloat start_x;
+  gfloat start_y;
   gfloat x;
   gfloat y;
   guint updated : 1;
@@ -231,6 +233,7 @@ meta_wayland_touch_update (MetaWaylandTouch   *touch,
 
       touch_info = touch_get_info (touch, sequence, TRUE);
       touch_info->touch_surface = touch_surface_get (touch, surface);
+      clutter_event_get_coords (event, &touch_info->start_x, &touch_info->start_y);
     }
   else
     touch_info = touch_get_info (touch, sequence, FALSE);
@@ -245,7 +248,7 @@ meta_wayland_touch_update (MetaWaylandTouch   *touch,
       struct wl_client *client = wl_resource_get_client (surface->resource);
       struct wl_display *display = wl_client_get_display (client);
 
-      touch_info->slot_serial = wl_display_next_serial (display);
+      touch_info->slot_serial = wl_display_get_serial (display);
     }
 
   touch_get_relative_coordinates (touch, touch_info->touch_surface->surface,
@@ -361,7 +364,7 @@ touch_send_frame_event (MetaWaylandTouch *touch)
 {
   GList *surfaces, *s;
 
-  surfaces = s = touch_get_surfaces (touch, TRUE);
+  surfaces = touch_get_surfaces (touch, TRUE);
 
   for (s = surfaces; s; s = s->next)
     {
@@ -444,15 +447,17 @@ touch_info_free (MetaWaylandTouchInfo *touch_info)
   g_free (touch_info);
 }
 
-static void
-touch_handle_cancel_event (MetaWaylandTouch      *touch,
-                           struct libinput_event *event)
+void
+meta_wayland_touch_cancel (MetaWaylandTouch *touch)
 {
   GList *surfaces, *s;
 
+  if (touch->display == NULL)
+    return;
+
   surfaces = s = touch_get_surfaces (touch, FALSE);
 
-  while (s)
+  for (s = surfaces; s; s = s->next)
     {
       MetaWaylandTouchSurface *touch_surface = s->data;
       struct wl_resource *resource;
@@ -495,7 +500,7 @@ evdev_filter_func (struct libinput_event *event,
        * which are not so useful when sending a global signal as the protocol
        * requires.
        */
-      touch_handle_cancel_event (touch, event);
+      meta_wayland_touch_cancel (touch);
       break;
     default:
       break;
@@ -529,8 +534,9 @@ void
 meta_wayland_touch_release (MetaWaylandTouch *touch)
 {
   clutter_evdev_remove_filter (evdev_filter_func, touch);
-  g_hash_table_unref (touch->touch_surfaces);
-  g_hash_table_unref (touch->touches);
+  g_clear_pointer (&touch->touch_surfaces, (GDestroyNotify) g_hash_table_unref);
+  g_clear_pointer (&touch->touches, (GDestroyNotify) g_hash_table_unref);
+  touch->display = NULL;
 }
 
 void
@@ -541,8 +547,63 @@ meta_wayland_touch_create_new_resource (MetaWaylandTouch   *touch,
 {
   struct wl_resource *cr;
 
-  cr = wl_resource_create (client, &wl_touch_interface,
-			   MIN (META_WL_TOUCH_VERSION, wl_resource_get_version (seat_resource)), id);
+  if (touch->display == NULL)
+    {
+      wl_resource_post_error (seat_resource, WL_DISPLAY_ERROR_INVALID_METHOD,
+                              "Cannot retrieve touch interface without touch capability");
+      return;
+    }
+
+  cr = wl_resource_create (client, &wl_touch_interface, wl_resource_get_version (seat_resource), id);
   wl_resource_set_implementation (cr, &touch_interface, touch, unbind_resource);
   wl_list_insert (&touch->resource_list, wl_resource_get_link (cr));
 }
+
+ClutterEventSequence *
+meta_wayland_touch_find_grab_sequence (MetaWaylandTouch   *touch,
+                                       MetaWaylandSurface *surface,
+                                       uint32_t            serial)
+{
+  MetaWaylandTouchInfo *touch_info;
+  ClutterEventSequence *sequence;
+  GHashTableIter iter;
+
+  if (!touch->touches)
+    return NULL;
+
+  g_hash_table_iter_init (&iter, touch->touches);
+
+  while (g_hash_table_iter_next (&iter, (gpointer*) &sequence,
+                                 (gpointer*) &touch_info))
+    {
+      if (touch_info->slot_serial == serial &&
+	  touch_info->touch_surface->surface == surface)
+        return sequence;
+    }
+
+  return NULL;
+}
+
+gboolean
+meta_wayland_touch_get_press_coords (MetaWaylandTouch     *touch,
+                                     ClutterEventSequence *sequence,
+                                     gfloat               *x,
+                                     gfloat               *y)
+{
+  MetaWaylandTouchInfo *touch_info;
+
+  if (!touch->touches)
+    return FALSE;
+
+  touch_info = g_hash_table_lookup (touch->touches, sequence);
+
+  if (!touch_info)
+    return FALSE;
+
+  if (x)
+    *x = touch_info->start_x;
+  if (y)
+    *y = touch_info->start_y;
+
+  return TRUE;
+}
