@@ -36,9 +36,10 @@ struct _MetaWindowWayland
 {
   MetaWindow parent;
 
-  gboolean has_saved_pos;
-  int saved_x;
-  int saved_y;
+  MetaWaylandSerial pending_configure_serial;
+  gboolean has_pending_move;
+  int pending_move_x;
+  int pending_move_y;
 
   int last_sent_width;
   int last_sent_height;
@@ -126,7 +127,8 @@ surface_state_changed (MetaWindow *window)
 
   meta_wayland_surface_configure_notify (window->surface,
                                          wl_window->last_sent_width,
-                                         wl_window->last_sent_height);
+                                         wl_window->last_sent_height,
+                                         &wl_window->pending_configure_serial);
 }
 
 static void
@@ -158,7 +160,7 @@ meta_window_wayland_move_resize_internal (MetaWindow                *window,
                                           MetaMoveResizeResultFlags *result)
 {
   MetaWindowWayland *wl_window = META_WINDOW_WAYLAND (window);
-  gboolean should_move = FALSE;
+  gboolean can_move_now;
 
   g_assert (window->frame == NULL);
 
@@ -186,44 +188,35 @@ meta_window_wayland_move_resize_internal (MetaWindow                *window,
           window->rect.height = unconstrained_rect.height;
         }
 
-      CoglTexture *texture = window->surface->buffer->texture;
-      int new_buffer_width = cogl_texture_get_width (texture);
-      int new_buffer_height = cogl_texture_get_height (texture);
-      if (window->buffer_rect.width != new_buffer_width ||
-          window->buffer_rect.height != new_buffer_height)
-        {
-          *result |= META_MOVE_RESIZE_RESULT_RESIZED;
-          window->buffer_rect.width = new_buffer_width;
-          window->buffer_rect.height = new_buffer_height;
-        }
-
       /* This is a commit of an attach. We should move the window to match the
        * new position the client wants. */
-      should_move = TRUE;
-    }
-
-  if (constrained_rect.width != window->rect.width ||
-      constrained_rect.height != window->rect.height)
-    {
-      wl_window->has_saved_pos = TRUE;
-      wl_window->saved_x = constrained_rect.x;
-      wl_window->saved_y = constrained_rect.y;
-
-      wl_window->last_sent_width = constrained_rect.width;
-      wl_window->last_sent_height = constrained_rect.height;
-
-      meta_wayland_surface_configure_notify (window->surface,
-                                             constrained_rect.width,
-                                             constrained_rect.height);
+      can_move_now = TRUE;
     }
   else
     {
-      /* We're just moving the window, so we don't need to wait for a configure
-       * and then ack to simply move the window. */
-      should_move = TRUE;
+      if (constrained_rect.width != window->rect.width ||
+          constrained_rect.height != window->rect.height)
+        {
+          wl_window->last_sent_width = constrained_rect.width;
+          wl_window->last_sent_height = constrained_rect.height;
+
+          meta_wayland_surface_configure_notify (window->surface,
+                                                 constrained_rect.width,
+                                                 constrained_rect.height,
+                                                 &wl_window->pending_configure_serial);
+
+          /* We need to wait until the resize completes before we can move */
+          can_move_now = FALSE;
+        }
+      else
+        {
+          /* We're just moving the window, so we don't need to wait for a configure
+           * and then ack to simply move the window. */
+          can_move_now = TRUE;
+        }
     }
 
-  if (should_move)
+  if (can_move_now)
     {
       int new_x = constrained_rect.x;
       int new_y = constrained_rect.y;
@@ -245,6 +238,18 @@ meta_window_wayland_move_resize_internal (MetaWindow                *window,
           window->buffer_rect.y = new_buffer_y;
         }
     }
+  else
+    {
+      int new_x = constrained_rect.x;
+      int new_y = constrained_rect.y;
+
+      if (new_x != window->rect.x || new_y != window->rect.y)
+        {
+          wl_window->has_pending_move = TRUE;
+          wl_window->pending_move_x = new_x;
+          wl_window->pending_move_y = new_y;
+        }
+    }
 }
 
 static void
@@ -343,16 +348,40 @@ meta_window_wayland_new (MetaDisplay        *display,
   return window;
 }
 
+static gboolean
+should_do_pending_move (MetaWindowWayland *wl_window,
+                        MetaWaylandSerial *acked_configure_serial)
+{
+  if (!wl_window->has_pending_move)
+    return FALSE;
+
+  if (wl_window->pending_configure_serial.set)
+    {
+      /* If we're waiting for a configure and this isn't an ACK for
+       * any configure, then fizzle it out. */
+      if (!acked_configure_serial->set)
+        return FALSE;
+
+      /* If we're waiting for a configure and this isn't an ACK for
+       * the configure we're waiting for, then fizzle it out. */
+      if (acked_configure_serial->value != wl_window->pending_configure_serial.value)
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
 /**
  * meta_window_move_resize_wayland:
  *
  * Complete a resize operation from a wayland client.
  */
 void
-meta_window_wayland_move_resize (MetaWindow    *window,
-                                 MetaRectangle  new_geom,
-                                 int            dx,
-                                 int            dy)
+meta_window_wayland_move_resize (MetaWindow        *window,
+                                 MetaWaylandSerial *acked_configure_serial,
+                                 MetaRectangle      new_geom,
+                                 int                dx,
+                                 int                dy)
 {
   MetaWindowWayland *wl_window = META_WINDOW_WAYLAND (window);
   int gravity;
@@ -368,11 +397,11 @@ meta_window_wayland_move_resize (MetaWindow    *window,
   /* x/y are ignored when we're doing interactive resizing */
   if (!meta_grab_op_is_resizing (window->display->grab_op))
     {
-      if (wl_window->has_saved_pos)
+      if (wl_window->has_pending_move && should_do_pending_move (wl_window, acked_configure_serial))
         {
-          rect.x = wl_window->saved_x;
-          rect.y = wl_window->saved_y;
-          wl_window->has_saved_pos = FALSE;
+          rect.x = wl_window->pending_move_x;
+          rect.y = wl_window->pending_move_y;
+          wl_window->has_pending_move = FALSE;
           flags |= META_IS_MOVE_ACTION;
         }
       else
@@ -389,6 +418,8 @@ meta_window_wayland_move_resize (MetaWindow    *window,
         }
     }
 
+  wl_window->pending_configure_serial.set = FALSE;
+
   rect.width = new_geom.width;
   rect.height = new_geom.height;
 
