@@ -916,40 +916,6 @@ handle_input_xevent (MetaDisplay *display,
   return TRUE;
 }
 
-static void
-reload_xkb_rules (MetaScreen  *screen)
-{
-  MetaWaylandCompositor *compositor;
-  char **names;
-  int n_names;
-  gboolean ok;
-  const char *rules, *model, *layout, *variant, *options;
-
-  compositor = meta_wayland_compositor_get_default ();
-
-  ok = meta_prop_get_latin1_list (screen->display, screen->xroot,
-                                  screen->display->atom__XKB_RULES_NAMES,
-                                  &names, &n_names);
-  if (!ok)
-    return;
-
-  if (n_names != 5)
-    goto out;
-
-  rules = names[0];
-  model = names[1];
-  layout = names[2];
-  variant = names[3];
-  options = names[4];
-
-  meta_wayland_keyboard_set_keymap_names (&compositor->seat->keyboard,
-                                          rules, model, layout, variant, options,
-                                          META_WAYLAND_KEYBOARD_SKIP_XCLIENTS);
-
- out:
-  g_strfreev (names);
-}
-
 static void
 process_request_frame_extents (MetaDisplay    *display,
                                XEvent         *event)
@@ -1459,10 +1425,6 @@ handle_other_xevent (MetaDisplay *display,
             else if (event->xproperty.atom ==
                      display->atom__NET_DESKTOP_NAMES)
               meta_screen_update_workspace_names (display->screen);
-            else if (meta_is_wayland_compositor () &&
-                     event->xproperty.atom ==
-                     display->atom__XKB_RULES_NAMES)
-              reload_xkb_rules (display->screen);
 
             /* we just use this property as a sentinel to avoid
              * certain race conditions.  See the comment for the
@@ -1486,12 +1448,15 @@ handle_other_xevent (MetaDisplay *display,
     case ClientMessage:
       if (window)
         {
+#ifdef HAVE_WAYLAND
           if (event->xclient.message_type == display->atom_WL_SURFACE_ID)
             {
               guint32 surface_id = event->xclient.data.l[0];
               meta_xwayland_handle_wl_surface_id (window, surface_id);
             }
-          else if (!frame_was_receiver)
+          else
+#endif
+          if (!frame_was_receiver)
             meta_window_x11_client_message (window, event);
         }
       else
@@ -1614,12 +1579,10 @@ handle_other_xevent (MetaDisplay *display,
             /* Let XLib know that there is a new keyboard mapping.
              */
             XRefreshKeyboardMapping (&event->xmapping);
-            meta_display_process_mapping_event (display, event);
           }
       }
       break;
     default:
-#ifdef HAVE_XKB
       if (event->type == display->xkb_base_event_type)
         {
           XkbAnyEvent *xkb_ev = (XkbAnyEvent *) event;
@@ -1634,14 +1597,10 @@ handle_other_xevent (MetaDisplay *display,
                   meta_bell_notify (display, xkb_ev);
                 }
               break;
-            case XkbNewKeyboardNotify:
-            case XkbMapNotify:
-              if (xkb_ev->device == META_VIRTUAL_CORE_KEYBOARD_ID)
-                meta_display_process_mapping_event (display, event);
+            default:
               break;
             }
         }
-#endif
       break;
     }
 
